Ext.override(Ext.chart.axis.Axis, {
    calcEnds: function() {
        var me = this,
            math = Math,
            mmax = math.max,
            mmin = math.min,
            store = me.chart.substore || me.chart.store,
            series = me.chart.series.items,
            fields = me.fields,
            ln = fields.length,
            min = isNaN(me.minimum) ? Infinity : me.minimum,
            max = isNaN(me.maximum) ? -Infinity : me.maximum,
            prevMin = me.prevMin,
            prevMax = me.prevMax,
            aggregate = false,
            total = 0,
            excludes = [],
            outfrom, outto,
            i, l, values, rec, out;

        //if one series is stacked I have to aggregate the values
        //for the scale.
        for (i = 0, l = series.length; !aggregate && i < l; i++) {
            aggregate = aggregate || series[i].stacked;
            excludes = series[i].__excludes || excludes;
        }
        store.each(function(record) {
            if (aggregate) {
                if (!isFinite(min)) {
                    min = 0;
                }
                for (values = [0, 0], i = 0; i < ln; i++) {
                    if (excludes[i]) {
                        continue;
                    }
                    rec = record.get(fields[i]);
                    values[+(rec > 0)] += math.abs(rec);
                }
                max = mmax(max, -values[0], values[1]);
                min = mmin(min, -values[0], values[1]);
            }
            else {
                for (i = 0; i < ln; i++) {
                    if (excludes[i]) {
                        continue;
                    }
                    value = record.get(fields[i]);
                    max = mmax(max, value);
                    min = mmin(min, value);
                }
            }
        });
        if (!isFinite(max)) {
            max = me.prevMax || 0;
        }
        if (!isFinite(min)) {
            min = me.prevMin || 0;
        }
        //normalize min max for snapEnds.
        if (min != max && (max != (max >> 0))) {
            max = (max >> 0) + 1;
        }
        /*FIX: ADDED BY ARTYOM TO FIX CHART 0 PROBLEM*/
        if(max == min) {
            max += 1;
        }

        if (me.chart.series.items && me.chart.series.items[0]) {
            var series = me.chart.series.items[0];

            if (me.chart.series.items[0].label.display == 'center' && series.bbox && series.imposion){
                var scale = (series.column ? series.bbox.height - series.yPadding * 2 : series.bbox.width - series.xPadding * 2) / (max - min);
                if (series.xyLabelMargin){
                    max += Math.ceil(series.xyLabelMargin / scale);
                    this.xyLabelMargin = null;
                }
            }
        }
        /*-FIX*/
        out = Ext.draw.Draw.snapEnds(min, max, me.majorTickSteps !== false ?  (me.majorTickSteps +1) : me.steps);
        outfrom = out.from;
        outto = out.to;
        if (me.forceMinMax) {
            if (!isNaN(max)) {
                out.to = max;
            }
            if (!isNaN(min)) {
                out.from = min;
            }
        }
        if (!isNaN(me.maximum)) {
            //TODO(nico) users are responsible for their own minimum/maximum values set.
            //Clipping should be added to remove lines in the chart which are below the axis.
            out.to = me.maximum;
        }
        if (!isNaN(me.minimum)) {
            //TODO(nico) users are responsible for their own minimum/maximum values set.
            //Clipping should be added to remove lines in the chart which are below the axis.
            out.from = me.minimum;
        }

        //Adjust after adjusting minimum and maximum
        out.step = (out.to - out.from) / (outto - outfrom) * out.step;

        if (me.adjustMaximumByMajorUnit) {
            out.to += out.step;
        }
        if (me.adjustMinimumByMajorUnit) {
            out.from -= out.step;
        }
        me.prevMin = min == max? 0 : min;
        me.prevMax = max;
        return out;
    }
});


Ext.override(Ext.chart.series.Line, {
    drawSeries: function() {
        var me = this,
            chart = me.chart,
            store = chart.substore || chart.store,
            surface = chart.surface,
            chartBBox = chart.chartBBox,
            bbox = {},
            group = me.group,
            gutterX = chart.maxGutter[0],
            gutterY = chart.maxGutter[1],
            showMarkers = me.showMarkers,
            markerGroup = me.markerGroup,
            enableShadows = chart.shadow,
            shadowGroups = me.shadowGroups,
            shadowAttributes = me.shadowAttributes,
            smooth = me.smooth,
            lnsh = shadowGroups.length,
            dummyPath = ["M"],
            path = ["M"],
            markerIndex = chart.markerIndex,
            axes = [].concat(me.axis),
            shadowGroup,
            shadowBarAttr,
            xValues = [],
            yValues = [],
            storeIndices = [],
            numericAxis = true,
            axisCount = 0,
            onbreak = false,
            markerStyle = me.markerStyle,
            seriesStyle = me.seriesStyle,
            seriesLabelStyle = me.seriesLabelStyle,
            colorArrayStyle = me.colorArrayStyle,
            colorArrayLength = colorArrayStyle && colorArrayStyle.length || 0,
            posHash = {
                'left': 'right',
                'right': 'left',
                'top': 'bottom',
                'bottom': 'top'
            },
            isNumber = Ext.isNumber,
            seriesIdx = me.seriesIdx, shadows, shadow, shindex, fromPath, fill, fillPath, rendererAttributes,
            x, y, prevX, prevY, firstY, markerCount, i, j, ln, axis, ends, marker, markerAux, item, xValue,
            yValue, coords, xScale, yScale, minX, maxX, minY, maxY, line, animation, endMarkerStyle,
            endLineStyle, type, props, firstMarker, count, smoothPath, renderPath;

        //if store is empty then there's nothing to draw.
        if (!store || !store.getCount()) {
            return;
        }

        //prepare style objects for line and markers
        endMarkerStyle = Ext.apply(markerStyle, me.markerConfig);
        type = endMarkerStyle.type;
        delete endMarkerStyle.type;
        endLineStyle = Ext.apply(seriesStyle, me.style);
        //if no stroke with is specified force it to 0.5 because this is
        //about making *lines*
        if (!endLineStyle['stroke-width']) {
            endLineStyle['stroke-width'] = 0.5;
        }
        //If we're using a time axis and we need to translate the points,
        //then reuse the first markers as the last markers.
        if (markerIndex && markerGroup && markerGroup.getCount()) {
            for (i = 0; i < markerIndex; i++) {
                marker = markerGroup.getAt(i);
                markerGroup.remove(marker);
                markerGroup.add(marker);
                markerAux = markerGroup.getAt(markerGroup.getCount() - 2);
                marker.setAttributes({
                    x: 0,
                    y: 0,
                    translate: {
                        x: markerAux.attr.translation.x,
                        y: markerAux.attr.translation.y
                    }
                }, true);
            }
        }

        me.unHighlightItem();
        me.cleanHighlights();

        me.setBBox();
        bbox = me.bbox;

        me.clipRect = [bbox.x, bbox.y, bbox.width, bbox.height];

        chart.axes.each(function(axis) {
            //only apply position calculations to axes that affect this series
            //this means the axis in the position referred by this series and also
            //the axis in the other coordinate for this series. For example: (left, top|bottom),
            //or (top, left|right), etc.
            if (axis.position == me.axis || axis.position != posHash[me.axis]) {
                axisCount++;
                if (axis.type != 'Numeric') {
                    numericAxis = false;
                    return;
                }
                numericAxis = (numericAxis && axis.type == 'Numeric');
                if (axis) {
                    ends = axis.calcEnds();
                    if (axis.position == 'top' || axis.position == 'bottom') {
                        minX = ends.from;
                        maxX = ends.to;
                    }
                    else {
                        minY = ends.from;
                        maxY = ends.to;
                    }
                }
            }
        });

        //If there's only one axis specified for a series, then we set the default type of the other
        //axis to a category axis. So in this case numericAxis, which would be true if both axes affecting
        //the series are numeric should be false.
        if (numericAxis && axisCount == 1) {
            numericAxis = false;
        }

        // If a field was specified without a corresponding axis, create one to get bounds
        //only do this for the axis where real values are bound (that's why we check for
        //me.axis)
        if (me.xField && !isNumber(minX)) {
            if (me.axis == 'bottom' || me.axis == 'top') {
                axis = Ext.create('Ext.chart.axis.Axis', {
                    chart: chart,
                    fields: [].concat(me.xField)
                }).calcEnds();
                minX = axis.from;
                maxX = axis.to;
            } else if (numericAxis) {
                axis = Ext.create('Ext.chart.axis.Axis', {
                    chart: chart,
                    fields: [].concat(me.xField),
                    forceMinMax: true
                }).calcEnds();
                minX = axis.from;
                maxX = axis.to;
            }
        }

        if (me.yField && !isNumber(minY)) {
            if (me.axis == 'right' || me.axis == 'left') {
                axis = Ext.create('Ext.chart.axis.Axis', {
                    chart: chart,
                    fields: [].concat(me.yField)
                }).calcEnds();
                minY = axis.from;
                maxY = axis.to;
            } else if (numericAxis) {
                axis = Ext.create('Ext.chart.axis.Axis', {
                    chart: chart,
                    fields: [].concat(me.yField),
                    forceMinMax: true
                }).calcEnds();
                minY = axis.from;
                maxY = axis.to;
            }
        }

        if (isNaN(minX)) {
            minX = 0;
            xScale = bbox.width / (store.getCount() - 1);
        }
        else {
            //In case some person decides to set an axis' minimum and maximum
            //configuration properties to the same value, then fallback the
            //denominator to a > 0 value.
            xScale = bbox.width / ((maxX - minX) || (store.getCount() - 1));
        }

        if (isNaN(minY)) {
            minY = 0;
            yScale = bbox.height / (store.getCount() - 1);
        }
        else {
            //In case some person decides to set an axis' minimum and maximum
            //configuration properties to the same value, then fallback the
            //denominator to a > 0 value.
            yScale = bbox.height / ((maxY - minY) || (store.getCount() - 1));
        }

        store.each(function(record, i) {
            xValue = record.get(me.xField);
            yValue = record.get(me.yField);
            //skip undefined values
            if (typeof yValue == 'undefined' || (typeof yValue == 'string' && !yValue)) {
                //<debug warn>
                if (Ext.isDefined(Ext.global.console)) {
                    Ext.global.console.warn("[Ext.chart.series.Line]  Skipping a store element with an undefined value at ", record, xValue, yValue);
                }
                //</debug>
                return;
            }
            // Ensure a value
            if (typeof xValue == 'string' || typeof xValue == 'object'
                //set as uniform distribution if the axis is a category axis.
                || (me.axis != 'top' && me.axis != 'bottom' && !numericAxis)) {
                xValue = i;
            }
            if (typeof yValue == 'string' || typeof yValue == 'object'
                //set as uniform distribution if the axis is a category axis.
                || (me.axis != 'left' && me.axis != 'right' && !numericAxis)) {
                yValue = i;
            }
            storeIndices.push(i);
            xValues.push(xValue);
            yValues.push(yValue);
        }, me);

        ln = xValues.length;
        if (ln > bbox.width) {
            coords = me.shrink(xValues, yValues, bbox.width);
            xValues = coords.x;
            yValues = coords.y;
        }

        me.items = [];

        count = 0;
        ln = xValues.length;
        for (i = 0; i < ln; i++) {
            xValue = xValues[i];
            yValue = yValues[i];
            if (yValue === false) {
                if (path.length == 1) {
                    path = [];
                }
                onbreak = true;
                me.items.push(false);
                continue;
            } else {
                x = (bbox.x + (xValue - minX) * xScale).toFixed(2);
                y = ((bbox.y + bbox.height) - (yValue - minY) * yScale).toFixed(2);
                if (onbreak) {
                    onbreak = false;
                    path.push('M');
                }
                path = path.concat([x, y]);
            }
            if ((typeof firstY == 'undefined') && (typeof y != 'undefined')) {
                firstY = y;
            }
            // If this is the first line, create a dummypath to animate in from.
            if (!me.line || chart.resizing) {
                dummyPath = dummyPath.concat([x, bbox.y + bbox.height / 2]);
            }

            // When resizing, reset before animating
            if (chart.animate && chart.resizing && me.line) {
                me.line.setAttributes({
                    path: dummyPath
                }, true);
                if (me.fillPath) {
                    me.fillPath.setAttributes({
                        path: dummyPath,
                        opacity: 0.2
                    }, true);
                }
                if (me.line.shadows) {
                    shadows = me.line.shadows;
                    for (j = 0, lnsh = shadows.length; j < lnsh; j++) {
                        shadow = shadows[j];
                        shadow.setAttributes({
                            path: dummyPath
                        }, true);
                    }
                }
            }
            if (showMarkers) {
                marker = markerGroup.getAt(count++);
                if (!marker) {
                    marker = Ext.chart.Shape[type](surface, Ext.apply({
                        group: [group, markerGroup],
                        x: 0, y: 0,
                        translate: {
                            x: prevX || x,
                            y: prevY || (bbox.y + bbox.height / 2)
                        },
                        value: '"' + xValue + ', ' + yValue + '"'
                    }, endMarkerStyle));
                    marker._to = {
                        translate: {
                            x: x,
                            y: y
                        }
                    };
                } else {
                    marker.setAttributes({
                        value: '"' + xValue + ', ' + yValue + '"',
                        x: 0, y: 0,
                        hidden: false
                    }, true);
                    marker._to = {
                        translate: {
                            x: x, y: y
                        }
                    };
                }
            }

            me.items.push({
                series: me,
                value: [xValue, yValue],
                point: [x, y],
                sprite: marker,
                storeItem: store.getAt(storeIndices[i])
            });
            prevX = x;
            prevY = y;
        }

        if (path.length <= 1) {
            //nothing to be rendered
            return;
        }

        if (smooth) {
            smoothPath = Ext.draw.Draw.smooth(path, isNumber(smooth) ? smooth : me.defaultSmoothness);
        }

        renderPath = smooth ? smoothPath : path;

        //Correct path if we're animating timeAxis intervals
        if (chart.markerIndex && me.previousPath) {
            fromPath = me.previousPath;
            if (!smooth) {
                Ext.Array.erase(fromPath, 1, 2);
            }
        } else {
            fromPath = path;
        }

        // Only create a line if one doesn't exist.
        if (!me.line) {
            me.line = surface.add(Ext.apply({
                type: 'path',
                group: group,
                path: dummyPath,
                stroke: endLineStyle.stroke || endLineStyle.fill
            }, endLineStyle || {}));
            //unset fill here (there's always a default fill withing the themes).
            me.line.setAttributes({
                fill: 'none'
            });
            if (!endLineStyle.stroke && colorArrayLength) {
                me.line.setAttributes({
                    stroke: colorArrayStyle[seriesIdx % colorArrayLength]
                }, true);
            }
            if (enableShadows) {
                //create shadows
                shadows = me.line.shadows = [];
                for (shindex = 0; shindex < lnsh; shindex++) {
                    shadowBarAttr = shadowAttributes[shindex];
                    shadowBarAttr = Ext.apply({}, shadowBarAttr, { path: dummyPath });
                    shadow = chart.surface.add(Ext.apply({}, {
                        type: 'path',
                        group: shadowGroups[shindex]
                    }, shadowBarAttr));
                    shadows.push(shadow);
                }
            }
        }
        if (me.fill) {
            fillPath = renderPath.concat([
                ["L", x, bbox.y + bbox.height],
                ["L", bbox.x, bbox.y + bbox.height],
                ["L", bbox.x, firstY]
            ]);
            if (!me.fillPath) {
                me.fillPath = surface.add({
                    group: group,
                    type: 'path',
                    opacity: endLineStyle.opacity || 0.3,
                    fill: endLineStyle.fill || colorArrayStyle[seriesIdx % colorArrayLength],
                    path: dummyPath
                });
            }
        }
        markerCount = showMarkers && markerGroup.getCount();
        if (chart.animate) {
            fill = me.fill;
            line = me.line;
            //Add renderer to line. There is not unique record associated with this.
            rendererAttributes = me.renderer(line, false, { path: renderPath }, i, store);
            Ext.apply(rendererAttributes, endLineStyle || {}, {
                stroke: endLineStyle.stroke || endLineStyle.fill
            });
            //fill should not be used here but when drawing the special fill path object
            delete rendererAttributes.fill;
            if (chart.markerIndex && me.previousPath) {
                me.animation = animation = me.onAnimate(line, {
                    to: rendererAttributes,
                    from: {
                        path: fromPath
                    }
                });
            } else {
                me.animation = animation = me.onAnimate(line, {
                    to: rendererAttributes
                });
            }
            //animate shadows
            if (enableShadows) {
                shadows = line.shadows;
                for(j = 0; j < lnsh; j++) {
                    if (chart.markerIndex && me.previousPath) {
                        me.onAnimate(shadows[j], {
                            to: { path: renderPath },
                            from: { path: fromPath }
                        });
                    } else {
                        me.onAnimate(shadows[j], {
                            to: { path: renderPath }
                        });
                    }
                }
            }
            //animate fill path
            if (fill) {
                me.onAnimate(me.fillPath, {
                    to: Ext.apply({}, {
                        path: fillPath,
                        fill: endLineStyle.fill || colorArrayStyle[seriesIdx % colorArrayLength]
                    }, endLineStyle || {})
                });
            }
            //animate markers
            if (showMarkers) {
                count = 0;
                for(i = 0; i < ln; i++) {
                    if (me.items[i]) {
                        item = markerGroup.getAt(count++);
                        if (item) {
                            rendererAttributes = me.renderer(item, store.getAt(i), item._to, i, store);
                            me.onAnimate(item, {
                                to: Ext.apply(rendererAttributes, endMarkerStyle || {})
                            });
                        }
                    }
                }
                for(; count < markerCount; count++) {
                    item = markerGroup.getAt(count);
                    item.hide(true);
                }
            }
        } else {

            //Set hidden to true
            rendererAttributes = me.renderer(me.line, false, { path: renderPath, hidden: true }, i, store);
            Ext.apply(rendererAttributes, endLineStyle || {}, {
                stroke: endLineStyle.stroke || endLineStyle.fill
            });
            //fill should not be used here but when drawing the special fill path object
            delete rendererAttributes.fill;
            me.line.setAttributes(rendererAttributes, true);
            //set path for shadows
            if (enableShadows) {
                shadows = me.line.shadows;
                for(j = 0; j < lnsh; j++) {
                    shadows[j].setAttributes({
                        path: renderPath
                    }, true);
                }
            }
            if (me.fill) {
                me.fillPath.setAttributes({
                    path: fillPath
                }, true);
            }
            if (showMarkers) {
                count = 0;
                for(i = 0; i < ln; i++) {
                    if (me.items[i]) {
                        item = markerGroup.getAt(count++);
                        if (item) {
                            rendererAttributes = me.renderer(item, store.getAt(i), item._to, i, store);
                            item.setAttributes(Ext.apply(endMarkerStyle || {}, rendererAttributes || {}), true);
                        }
                    }
                }
                for(; count < markerCount; count++) {
                    item = markerGroup.getAt(count);
                    item.hide(true);
                }
            }
        }

        if (chart.markerIndex) {
            if (me.smooth) {
                Ext.Array.erase(path, 1, 2);
            } else {
                Ext.Array.splice(path, 1, 0, path[1], path[2]);
            }
            me.previousPath = path;
        }
        me.renderLabels();
        me.renderCallouts();
    }
});

Ext.override(Ext.chart.series.Bar, {
     getPaths: function() {
        var me = this,
            chart = me.chart,
            store = chart.substore || chart.store,
            bounds = me.bounds = me.getBounds(),
            items = me.items = [],
            gutter = me.gutter / 100,
            groupGutter = me.groupGutter / 100,
            animate = chart.animate,
            column = me.column,
            group = me.group,
            enableShadows = chart.shadow,
            shadowGroups = me.shadowGroups,
            shadowAttributes = me.shadowAttributes,
            shadowGroupsLn = shadowGroups.length,
            bbox = bounds.bbox,
            xPadding = me.xPadding,
            yPadding = me.yPadding,
            stacked = me.stacked,
            barsLen = bounds.barsLen,
            colors = me.colorArrayStyle,
            colorLength = colors && colors.length || 0,
            math = Math,
            mmax = math.max,
            mmin = math.min,
            mabs = math.abs,
            j, yValue, height, totalDim, totalNegDim, bottom, top, hasShadow, barAttr, attrs, counter,
            shadowIndex, shadow, sprite, offset, floorY;

        store.each(function(record, i, total) {
            bottom = bounds.zero;
            top = bounds.zero;
            totalDim = 0;
            totalNegDim = 0;
            hasShadow = false;
            for (j = 0, counter = 0; j < barsLen; j++) {
                // Excluded series
                if (me.__excludes && me.__excludes[j]) {
                    continue;
                }
                yValue = record.get(bounds.bars[j]);
                height = Math.round((yValue - ((bounds.minY < 0) ? 0 : bounds.minY)) * bounds.scale);
                barAttr = {
                    fill: colors[(barsLen > 1 ? j : 0) % colorLength]
                };
                if (column) {
                    Ext.apply(barAttr, {
                        height: height,
                        width: mmax(bounds.groupBarWidth, 0),
                        x: (bbox.x + xPadding + i * bounds.barWidth * (1 + gutter) + counter * bounds.groupBarWidth * (1 + groupGutter) * !stacked),
                        y: bottom - height
                    });
                }
                else {
                    // draw in reverse order
                    offset = (total - 1) - i;
                    Ext.apply(barAttr, {
                        height: mmax(bounds.groupBarWidth, 0),
                        width: height + (bottom == bounds.zero),
                        x: bottom + (bottom != bounds.zero),
                        y: (bbox.y + yPadding + offset * bounds.barWidth * (1 + gutter) + counter * bounds.groupBarWidth * (1 + groupGutter) * !stacked + 1)
                    });
                }
                if (height < 0) {
                    if (column) {
                        barAttr.y = top;
                        barAttr.height = mabs(height);
                    } else {
                        barAttr.x = top + height;
                        barAttr.width = mabs(height);
                    }
                }
                if (stacked) {
                    if (height < 0) {
                        top += height * (column ? -1 : 1);
                    } else {
                        bottom += height * (column ? -1 : 1);
                    }
                    totalDim += mabs(height);
                    if (height < 0) {
                        totalNegDim += mabs(height);
                    }
                }
                barAttr.x = Math.floor(barAttr.x) + 1;
                floorY = Math.floor(barAttr.y);
                if (!Ext.isIE9 && barAttr.y > floorY) {
                    floorY--;
                }
                barAttr.y = floorY;
                barAttr.width = Math.floor(barAttr.width);
                barAttr.height = Math.floor(barAttr.height);
                items.push({
                    series: me,
                    storeItem: record,
                    value: [record.get(me.xField), yValue],
                    attr: barAttr,
                    point: column ? [barAttr.x + barAttr.width / 2, yValue >= 0 ? barAttr.y : barAttr.y + barAttr.height] :
                                    [yValue >= 0 ? barAttr.x + barAttr.width : barAttr.x, barAttr.y + barAttr.height / 2],
                    _yField: bounds.bars[j]
                });
                // When resizing, reset before animating
                if (animate && chart.resizing) {
                    attrs = column ? {
                        x: barAttr.x,
                        y: bounds.zero,
                        width: barAttr.width,
                        height: 0
                    } : {
                        x: bounds.zero,
                        y: barAttr.y,
                        width: 0,
                        height: barAttr.height
                    };
                    if (enableShadows && (stacked && !hasShadow || !stacked)) {
                        hasShadow = true;
                        //update shadows
                        for (shadowIndex = 0; shadowIndex < shadowGroupsLn; shadowIndex++) {
                            shadow = shadowGroups[shadowIndex].getAt(stacked ? i : (i * barsLen + j));
                            if (shadow) {
                                shadow.setAttributes(attrs, true);
                            }
                        }
                    }
                    //update sprite position and width/height
                    sprite = group.getAt(i * barsLen + j);
                    if (sprite) {
                        sprite.setAttributes(attrs, true);
                    }
                }
                counter++;
            }
            if (stacked && items.length) {
                items[i * counter].totalDim = totalDim;
                items[i * counter].totalNegDim = totalNegDim;
            }
        }, me);
    },

    onPlaceLabel: function(label, storeItem, item, i, display, animate, j, index) {
                // Determine the label's final position. Starts with the configured preferred value but
                // may get flipped from inside to outside or vice-versa depending on space.
                var me = this,
                    opt = me.bounds,
                    groupBarWidth = opt.groupBarWidth,
                    column = me.column,
                    chart = me.chart,
                    chartBBox = chart.chartBBox,
                    resizing = chart.resizing,
                    xValue = item.value[0],
                    yValue = item.value[1],
                    attr = item.attr,
                    config = me.label,
                    rotate = config.orientation == 'vertical',
                    field = [].concat(config.field),
                    format = config.renderer,
                    text = format(storeItem.get(field[index])),
                    size = me.getLabelSize(text),
                    width = size.width,
                    height = size.height,
                    zero = opt.zero,
                    outside = 'outside',
                    insideStart = 'insideStart',
                    insideEnd = 'insideEnd',
                    center = 'center',
                    offsetX = 10,
                    offsetY = 6,
                    signed = opt.signed,
                    x, y, finalAttr;

                label.setAttributes({
                    text: text,
                    hidden: true
                }, true);

                label.isOutside = false;
                if (column) {
                    if (display == outside) {
                        if (height + offsetY + attr.height > (yValue >= 0 ? zero - chartBBox.y : chartBBox.y + chartBBox.height - zero)) {
                            display = insideEnd;
                        }
                    } else {
                        if (height + offsetY > attr.height) {
                            display = outside;
                            label.isOutside = true;
                        }
                    }
                    x = attr.x + groupBarWidth / 2;
                    y = display == insideStart ?
                            (zero + ((height / 2 + 3) * (yValue >= 0 ? -1 : 1))) :
                            (yValue >= 0 ? (attr.y + ((height / 2 + 3) * (display == outside ? -1 : 1))) :
                                           (attr.y + attr.height + ((height / 2 + 3) * (display === outside ? 1 : -1))));
                    if (display == center) {
                        y = (  attr.y + attr.height/2  );
                    }
                }
                else {
                    if (display == outside) {
                        if (width + offsetX + attr.width > (yValue >= 0 ? chartBBox.x + chartBBox.width - zero : zero - chartBBox.x)) {
                            display = insideEnd;
                        }
                    }
                    else {
                        if (width + offsetX > attr.width) {
                            display = outside;
                            label.isOutside = true;
                        }
                    }
                    x = display == insideStart ?
                        (zero + ((width / 2 + 5) * (yValue >= 0 ? 1 : -1))) :
                        (yValue >= 0 ? (attr.x + attr.width + ((width / 2 + 5) * (display === outside ? 1 : -1))) :
                        (attr.x + ((width / 2 + 5) * (display === outside ? -1 : 1))));
                    y = attr.y + groupBarWidth / 2;
                    if (display == center) {
                        x = ( attr.x + attr.width/2 );
                    }
                }
                //first check for imposition & posible to put staked labels
                if (!this.imposion)
                     this.imposion = [];
                var tryingStaked = 5,
                    isOverlap,
                    impositionLastX,
                    impositionLastY,
                    impositionLastW;
                while (!!(--tryingStaked) ) {
                    isOverlap = false;
                    for (var i = 0; i<me.imposion.length; i++){
                        if (
                                (x < me.imposion[i][0] + me.imposion[i][2] + (column?0:offsetX)) && (x+width > me.imposion[i][0])
                                &&
                                (y < me.imposion[i][1] + me.imposion[i][3]) && (y+height > me.imposion[i][1])
                           ){
                            isOverlap = true;
                            impositionLastX = me.imposion[i][0];
                            impositionLastY = me.imposion[i][1];
                            impositionLastW = me.imposion[i][2];
                        }
                    }
                    if (!isOverlap) {
                        break;
                    } else {
                        if (column)
                            y = impositionLastY - height - 1;
                        else
                            x = impositionLastX + impositionLastW/2 + offsetX + width/2 + 1;
                    }

                }
                //set position
                finalAttr = {
                    x: x,
                    y: y,
                    hidden: false
                };
                //rotate
                if (rotate) {
                    finalAttr.rotate = {
                        x: x,
                        y: y,
                        degrees: 270
                    };
                }
                //check for resizing
                if (animate && resizing) {
                    if (column) {
                        x = attr.x + attr.width / 2;
                        y = zero;
                    } else {
                        x = zero;
                        y = attr.y + attr.height / 2;
                    }
                    label.setAttributes({
                        x: x,
                        y: y
                    }, true);
                    if (rotate) {
                        label.setAttributes({
                            rotate: {
                                x: x,
                                y: y,
                                degrees: 270
                            }
                        }, true);
                    }
                }
                //Check for overlap                
                x = parseFloat(x);
                y = parseFloat(y);
                var isOverlap = false;
                for (var i = 0; i<me.imposion.length; i++){
                    if (
                            (x < me.imposion[i][0] + me.imposion[i][2]) && (x+width > me.imposion[i][0])
                            &&
                            (y < me.imposion[i][1] + me.imposion[i][3]) && (y+height > me.imposion[i][1])
                       ){
                        isOverlap = true;
                    }
                }
                //sometimes values are outside of the column chart screen on the top
                if (chartBBox.y > y-5 || chartBBox.x + chartBBox.width < x - offsetX){
                    if (column)
                        this.xyLabelMargin = Math.max(chartBBox.y - y + height + offsetY, this.xyLabelMargin||0);
                    else
                        this.xyLabelMargin = Math.max(x - chartBBox.x + chartBBox.width + width + offsetX, this.xyLabelMargin||0);

                    //do not show labels which out of the chart screen box
                    return label.setAttributes({hidden: true}, true);
                }

                if (!isOverlap)
                    me.imposion.push([x,y,width,height]);

                //handle animation
                if (animate) {
                    if (!isOverlap)
                    me.onAnimate(label, { to: finalAttr });
                }
                else {
                    if (!isOverlap)
                    label.setAttributes(Ext.apply(finalAttr, {
                        hidden: false
                    }), true);
                }
    }
    
});