Ext.define('App.proxy.Events', {
    extend: 'App.proxy.AbstractProxy',

    eventsQueue: [],
    _dropIndexRequested: false,
    _dropIndexRequestRecordCount: 499,
    _dirrectoryFetched: false,
    _dirrectoryUsers:[],
    _dirrectoryTruncs:[],
    _dirrectoryAreas:[],
    _dirrectoryTags:[],
    _dirrectoryPbxes: new Map(),
    _dirrectoryUsersMapLocalNUMBER: new Map(),
    _dirrectoryUsersMapRemoteNUMBER: new Map(),
    _dirrectoryUsersMapMOBILE_NUMBER: new Map(),
    _dirrectoryUsersMapFAX_NUMBER: new Map(),
    _dirrectoryUsersMapOFFICE_NUMBER: new Map(),
    _dirrectoryUsersMapHOME_NUMBER: new Map(),
    _dirrectoryUsersMapHOME_MOBILE_NUMBER: new Map(),
    _dirrectoryUsersMapPHONE_NUMBER: new Map(),
    _dirrectoryAreasMap:{},
    _userIds: new Set(),

    /**
     * Synchronize events to the local db
     * @param params {events, minDate, maxDate, minId, maxId}
     * event has fields: id,from,fromChannel,to,toChannel,type,class,direction,date,duration,billsec,cost,tags
     * @param {type} callback
     * @returns {undefined}
     */
    pushEvents: function(params, callback){
        var self = this,
            esc = self.escape,
            storage = self.getStorage(),
            items = params.events,
            minDate = params.minDate,
            maxDate = params.maxDate,
            isFirstPageOfEvents = !self._dirrectoryFetched,
            isLastPageOfEvents = items.length === 0;

        if (isFirstPageOfEvents){
            //First time
            if (items.length >= self._dropIndexRequestRecordCount){
                self._dropIndexRequested = true;
            }
        }
        if (isLastPageOfEvents){
            //Skip dirrectories loading if no events for insert
            self._dirrectoryFetched = true;
        } else if (!isLastPageOfEvents){
            if (self._dirrectoryFetched && self._dirrectoryUsers.length == 0){
                self._dirrectoryFetched = false;
            }
        }

        var externalUsers = [];
        var externalUsersNumbers = new Set();

        this.getDirrectories()
            .then(function(){
                if (self._dropIndexRequested && isFirstPageOfEvents){
                    return storage.dropIndexPromise();
                }
            })
            .then(function(){
                var d, userFrom, userTo, trunk, area, pbx, eventTags, tag;
                var stats = [];
                var statDay = 0,
                    statMinId = 0,
                    statMaxId = 0;
                var seq = Promise.resolve();
                for (var i=0; i<items.length; i++){
                    d = {};
                    // Set ID
                    d['ID'] = parseInt(items[i].id);
                    /**
                     * API returns destType
                     * dest_type = 2 (incoming call from trunk or remote trunk from wms network)
                     * dest_type = 4 (incoming call from local user)
                     * dest_type = 3 (outgoing call to trunk/wms network)
                     * dest_type = 0 (outgoing call to local user)
                     *
                     * calls from the trunk can be local eg
                     * the user is calling from his mobile to a local user (WMS-1452)
                     *
                     * callclass = 5 (outgoing call to emergency number)
                     *
                     * to protect wrong assotiations short numbers with local users we introduce logic with isEmergencyCall
                     * if isEmergencyCall we dont associate c_to with short number of colleagues    (WMS-1093)
                     */

                    var isEmergencyCall = items[i]['class'] == 5;
                    // Set FROM, FROM_USER, FROM_GROUP, TO, TO_USER, TO_GROUP
                    userFrom = self.getUserModelByNumber(items[i].from, false);
                    userTo = self.getUserModelByNumber(items[i].to, isEmergencyCall);

                    var userData = null;
                    var numberFrom = items[i].from.replace('+','');
                    var numberTo = items[i].to.replace('+','');

                    if (!userFrom && !externalUsersNumbers.has(numberFrom)) {
                        externalUsersNumbers.add(numberFrom);
                        userData = self.generateUserData(items[i].from, items[i].fromName);
                    } else if (!userTo && !externalUsersNumbers.has(numberTo)) {
                        externalUsersNumbers.add(numberTo);
                        userData = self.generateUserData(items[i].to, items[i].toName);
                    }

                    if (userData) {
                        self.addExternalUserModel(userData);
                        externalUsers.push(userData);
                    }

                    d['FROM'] = items[i].from;
                    d['FROM_USER'] = userFrom ? userFrom['ID'] : (userData ? userData.id : null);
                    d['FROM_GROUP'] = userFrom?userFrom['GROUP_ID']:null;
                    d['TO'] = items[i].to;
                    d['TO_USER'] = userTo ? userTo['ID'] : (userData ? userData.id : null);
                    d['TO_GROUP'] = userTo?userTo['GROUP_ID']:null;
                    d['FROM_NAME'] = items[i].fromName;
                    d['TO_NAME'] = items[i].toName;
                    // Set TYPE, CLASS, DIRECTION
                    d['TYPE'] = items[i].type;
                    d['CLASS'] = items[i]['class'];
                    d['DIRECTION'] = items[i].direction;
                    // Set DATE
                    d['DATE'] = items[i].date;
                    // Set HOUR, DAY, MONTH, YEAR
                    d['HOUR'] = self.getHour(items[i].date);
                    d['DAY'] = self.getDay(items[i].date);
                    d['MONTH'] = self.getMonth(items[i].date);
                    d['YEAR'] = self.getYear(items[i].date);
                    // Set DURATION, BILL, COST
                    d['DURATION'] = items[i].duration;
                    d['BILL'] = parseInt(items[i].billsec);
                    d['COST'] = items[i].cost || 0;
                    // Set TRUNK
                    if (items[i].direction == 2) {
                        trunk = self.getTrunkModel( items[i].fromChannel );

                        if (trunk != null) {
                            d['TRUNK'] = trunk['ID'];
                        } else {
                            d['TRUNK'] = null;
                        }
                    } else if (items[i].direction == 3) {
                        trunk = self.getTrunkModel( items[i].toChannel );

                        if (trunk != null) {
                            d['TRUNK'] = trunk['ID'];
                        } else {
                            d['TRUNK'] = null;
                        }
                    } else {
                        d['TRUNK'] = null;
                    }

                    // SET PBX commented in applet, so skip here too
                    pbx = self.getPbxModel(items[i].pbx);
                    d['PBX'] = pbx ? pbx['ID'] : null;
                    // Set COUNTRY, COUNTRY_AREA
                    if (items[i].direction == 2) {
                        area = self.getAreaModel(items[i].from);
                        if (area != null) {
                            d['COUNTRY'] = area['COUNTRY_ID'];
                            d['COUNTRY_AREA'] = area['ID'];
                        } else {
                            d['COUNTRY'] = null;
                            d['COUNTRY_AREA'] = null;
                        }
                    } else if (items[i].direction == 3) {
                        area = self.getAreaModel(items[i].to);

                        if (area != null) {
                            d['COUNTRY'] = area['COUNTRY_ID'];
                            d['COUNTRY_AREA'] = area['ID'];
                        } else {
                            d['COUNTRY'] = null;
                            d['COUNTRY_AREA'] = null;
                        }
                    } else {
                        d['COUNTRY'] = null;
                        d['COUNTRY_AREA'] = null;
                    }
                    // Set EVENT_LOCAL - 23, EVENT_INCOMING - 24, EVENT_OUTGOING - 25, EVENT_ANSWERED - 26, EVENT_UNANSWERED - 27
                    if (items[i].direction == 2) {
                        d['EVENT_LOCAL'] = 0;
                        d['EVENT_INCOMING'] = 1;
                        d['EVENT_OUTGOING'] = 0;
                    } else if (items[i].direction == 3) {
                        d['EVENT_LOCAL'] = 0;
                        d['EVENT_INCOMING'] = 0;
                        d['EVENT_OUTGOING'] = 1;
                    } else if (items[i].direction == 4) {
                        d['EVENT_LOCAL'] = 1;
                        d['EVENT_INCOMING'] = 0;
                        d['EVENT_OUTGOING'] = 0;
                    } else {
                        d['EVENT_LOCAL'] = 0;
                        d['EVENT_INCOMING'] = 0;
                        d['EVENT_OUTGOING'] = 0;
                    }

                    if (items[i]['type'] == 3) {
                        d['EVENT_ANSWERED'] = 0;
                        d['EVENT_UNANSWERED'] = 1;
                    } else {
                        if (items[i].billsec > 0) {
                            d['EVENT_ANSWERED'] = 1;
                            d['EVENT_UNANSWERED'] = 0;
                        } else {
                            d['EVENT_ANSWERED'] = 0;
                            d['EVENT_UNANSWERED'] = 1;
                        }
                    }
                    // Set DURATION_LOCAL - 28, DURATION_INCOMING - 29, DURATION_OUTGOING - 30
                    if (items[i].direction == 2) {
                        d['DURATION_LOCAL'] = 0;
                        d['DURATION_INCOMING'] = items[i].duration;
                        d['DURATION_OUTGOING'] = 0;
                    } else if (items[i].direction == 3) {
                        d['DURATION_LOCAL'] = 0;
                        d['DURATION_INCOMING'] = 0;
                        d['DURATION_OUTGOING'] = items[i].duration;
                    } else if (items[i].direction == 4) {
                        d['DURATION_LOCAL'] = items[i].duration;
                        d['DURATION_INCOMING'] = 0;
                        d['DURATION_OUTGOING'] = 0;
                    } else {
                        d['DURATION_LOCAL'] = 0;
                        d['DURATION_INCOMING'] = 0;
                        d['DURATION_OUTGOING'] = 0;
                    }
                    // Set BILL_LOCAL - 31, BILL_INCOMING - 32, BILL_OUTGOING - 33
                    if (items[i].direction == 2) {
                        d['BILL_LOCAL'] = 0;
                        d['BILL_INCOMING'] = items[i].billsec;
                        d['BILL_OUTGOING'] = 0;
                    } else if (items[i].direction == 3) {
                        d['BILL_LOCAL'] = 0;
                        d['BILL_INCOMING'] = 0;
                        d['BILL_OUTGOING'] = items[i].billsec;
                    } else if (items[i].direction == 4) {
                        d['BILL_LOCAL'] = items[i].billsec;
                        d['BILL_INCOMING'] = 0;
                        d['BILL_OUTGOING'] = 0;
                    } else {
                        d['BILL_LOCAL'] = 0;
                        d['BILL_INCOMING'] = 0;
                        d['BILL_OUTGOING'] = 0;
                    }
                    // Set RING - 34
                    d['RING'] = items[i].duration - items[i].billsec;
                    // Set HOUR_OF_DAY - 35
                    var date = new Date((items[i].date || '').replace(' ','T'));
                    d['HOUR_OF_DAY'] = self.getHourOfDay(date);
                    // Set DAY_OF_WEEK - 36
                    d['DAY_OF_WEEK'] = self.getDayOfWeek(date);
                    // Set MONTH_OF_YEAR - 37
                    d['MONTH_OF_YEAR'] = self.getMonthOfYear(date);
                    // Set EVENT_CALL - 38, EVENT_VOICEMAIL - 39, EVENT_FAX - 40
                    if (items[i]['type'] == 1) {
                        d['EVENT_CALL'] = 1;
                        d['EVENT_VOICEMAIL'] = 0;
                        d['EVENT_FAX'] = 0;
                    } else if (items[i]['type'] == 3) {
                        d['EVENT_CALL'] = 0;
                        d['EVENT_VOICEMAIL'] = 1;
                        d['EVENT_FAX'] = 0;
                    } else if (items[i]['type'] == 2) {
                        d['EVENT_CALL'] = 0;
                        d['EVENT_VOICEMAIL'] = 0;
                        d['EVENT_FAX'] = 1;
                    } else {
                        d['EVENT_CALL'] = 0;
                        d['EVENT_VOICEMAIL'] = 0;
                        d['EVENT_FAX'] = 0;
                    }
                    // Set COST_CALL - 41, COST_VOICEMAIL - 42, COST_FAX - 43
                    if (items[i]['type'] == 1) {
                        d['COST_CALL'] = items[i].cost || 0;
                        d['COST_VOICEMAIL'] = 0;
                        d['COST_FAX'] = 0;
                    } else if (items[i]['type'] == 3) {
                        d['COST_CALL'] = 0;
                        d['COST_VOICEMAIL'] = items[i].cost || 0;
                        d['COST_FAX'] = 0;
                    } else if (items[i]['type'] == 2) {
                        d['COST_CALL'] = 0;
                        d['COST_VOICEMAIL'] = 0;
                        d['COST_FAX'] = items[i].cost || 0;
                    } else {
                        d['COST_CALL'] = 0;
                        d['COST_VOICEMAIL'] = 0;
                        d['COST_FAX'] = 0;
                    }
                    // Add tags for those event
                    eventTags = self.parseTags(items[i].tags);
                    if (eventTags.length>0){
                        for (var t=0;t<eventTags.length;t++){
                            tag = self.getTagModel(eventTags[t]);
                            if (!tag) {
                                tag = self.addTagModel(eventTags[t]);
                                seq = seq.then(Ext.bind(function(tagModel){
                                    return storage.bulkInsertPromise('TAGS',[{
                                            ID: null,
                                            NAME: tagModel['NAME']
                                    }]).then(function(result){
                                        tagModel['ID'] = result.lastInsertId;
                                    });
                                },self,[tag]));
                            }
                            if (!tag.eventIds)
                                tag.eventIds = [];
                            tag.eventIds.push(items[i].id);
                        }
                        d['TAGS'] = eventTags.join(', ');
                    } else {
                        d['TAGS'] = null;
                    }

                    d['USERFIELD'] = items[i].userfield;
                    d['FAXDATA'] = items[i].faxdata;

                    self.eventsQueue.push(d);
                    //collect statistics
                    if (statDay == 0) {
                        statDay = d['DAY'];
                        statMinId = d['ID'];
                        statMaxId = d['ID'];
                    } else {
                        if (statDay != d['DAY']) {
                            stats.push({
                                DAY:statDay,
                                MIN_ID:statMinId,
                                MAX_ID:statMaxId
                            });

                            statDay = 0;
                            statMinId = 0;
                            statMaxId = 0;
                        }

                        if (statDay != d['DAY']) {
                            statDay = d['DAY'];
                        }
                        if (d['ID'] < statMinId || statMinId == 0) {
                            statMinId = d['ID'];
                        }
                        if (d['ID'] > statMaxId || statMaxId == 0) {
                            statMaxId = d['ID'];
                        }
                    }

                }//end loop for new events
                //insert last statistic record after loop completed
                stats.push({
                    DAY:statDay,
                    MIN_ID:statMinId,
                    MAX_ID:statMaxId
                });

                seq = seq
                    .then(function(){
                        //Insert events
                        storage.bulkInsertPromise('EVENTS', self.eventsQueue);
                        if (externalUsers.length) {
                            return storage.pushUsers(externalUsers, false);
                        }
                    })
                    .then(function(){
                        //Ensert link between Event and Tag
                        var eventTagsInserts = [];
                        for (var i=0,l=self._dirrectoryTags.length;i<l;i++){
                            if (self._dirrectoryTags[i].eventIds){
                                self._dirrectoryTags[i].eventIds.reduce(function(eventTagsInserts, eventId){
                                    eventTagsInserts.push({EVENT_ID:eventId, TAG_ID: self._dirrectoryTags[i]['ID']});
                                    return eventTagsInserts;
                                },eventTagsInserts);
                            }
                            delete self._dirrectoryTags[i].eventIds;
                        }
                        return storage.bulkInsertPromise('EVENTS_TAGS',eventTagsInserts);
                    })
                    .then(function(){
                        //Get all previous stats
                        return storage.fetchPromise('SELECT ID, DAY ,DATE , MIN_ID , MAX_ID , COMPLETE FROM EVENTS_STATS');
                    })
                    .then(function(dbStats){
                        var statSeq = Promise.resolve();
                        var stat;
                        var needInsert, needUpdate;
                        var i, c, day;

                        var date;

                        //
                        var minDay =  self.getDay(minDate);
                        var maxDay =  self.getDay(maxDate);

                        // dbStats -- fetched from DB stats
                        // Verify that all days present in db
                        // if not we will add with null id and after we simle can decide if null than insert
                        for (day = minDay; day <= maxDay; day++) {

                            needInsert = true;
                            for (c=0; c < dbStats.length; c++) {
                                if (dbStats[c]['DAY'] == day) {
                                    needInsert = false;
                                    break;
                                }
                            }

    //                        console.log("[STATS] event (needInsert "+ needInsert +" ["+ day +"])");

                            if (needInsert) {
                                var time = new Date( day * 86400 * 1000);
                                date = time.getUTCFullYear()+'-'+sprintf('%02d',time.getUTCMonth()+1)+'-'+ sprintf('%02d',time.getUTCDate());
                                stat = {
                                    ID: null,
                                    MIN_ID: 0,
                                    MAX_ID: 0,
                                    DAY: day,
                                    DATE: date,
                                    COMPLETE:0
                                };


     //                           console.log("insert statistics for "+ date +" ("+ day +" day)");
                                dbStats.push(stat);
                            }
                        }

                        // Apply min and max ids
                        // Now if ID is null statistic will inserted with correct min and max id
                        // if not null we just update it
                        for (i=0; i < stats.length; i++) {

                            for (c=0; c < dbStats.length; c++) {
                                if (dbStats[c]['DAY'] == stats[i]['DAY']) {
                                    needUpdate = false;
                                    if (dbStats[c]['MIN_ID'] > stats[i]['MIN_ID'] || dbStats[c]['MIN_ID'] == 0) {
                                        dbStats[c]['MIN_ID'] = stats[i]['MIN_ID'];
                                        needUpdate = true;
                                    }

                                    if (dbStats[c]['MAX_ID'] < stats[i]['MAX_ID'] || dbStats[c]['MAX_ID'] == 0) {
                                        dbStats[c]['MAX_ID'] = stats[i]['MAX_ID'];
                                        needUpdate = true;
                                    }

                                    //updateEventStat
                                    if (needUpdate && dbStats[c]['ID'] != null){
                                        statSeq = statSeq.then(Ext.bind(storage.executePromise,storage,
                                            ['UPDATE `EVENTS_STATS` SET `MAX_ID`='+ esc( dbStats[c]['MAX_ID'] ) + ', '+
                                            ' `MIN_ID` = '+esc( dbStats[c]['MIN_ID'] )+' WHERE `ID` = '+ esc( dbStats[c]['ID'] )]));
                                    } else {
                                        //it will inserted with correct max & min ids
                                    }

                                    break;
                                }
                            }
                        }
                        //insert new one
                        //we are not wait for ID ! because next block will fetch from scratch
                        statSeq = statSeq.then(function(){
                            return storage.bulkInsertPromise('EVENTS_STATS',dbStats.filter(function(statistic){
                                    return statistic['ID'] == null;
                            }));
                        }).then(function(){
                            return storage.fetchPromise('SELECT ID, DAY ,DATE , MIN_ID , MAX_ID , COMPLETE FROM EVENTS_STATS');
                        }).then(function(dbStats){

                            // Set complete to all items, if client is insert an empty statistics.
    //                        console.log("insert statistics; eventsLength = "+ events.length +"");
                            if (isLastPageOfEvents) {
                                //Last fetched events always empty
                                var eventStatIds = [];
                                for (day = minDay; day <= maxDay; day++) {
                                    for (c=0; c < dbStats.length; c++) {
                                        if (dbStats[c]['DAY'] == day) {
                                            eventStatIds.push(esc( dbStats[c]['ID'] ));
                                        }
                                    }
                                }
                                if (eventStatIds.length>0){
                                    return storage.executePromise('UPDATE `EVENTS_STATS` SET `COMPLETE`= 1 WHERE `ID` IN ( '+ eventStatIds.join(',') +' )');
                                }
                            }
                            return;
                        });


                        return statSeq;
                    });
                return seq;
            })
            .then(function(){
                if (self._dropIndexRequested && isLastPageOfEvents){
                    return storage.createIndexPromise()
                        .then(function(){
                            self._dropIndexRequested = false;
                        });
                }
            })
            .then(callback)
            ['catch'](function(e){
                console.error('pushEvents error', self.getStorage()._lockedDbCounter, e);
                callback();
             });
    },
    addTagModel: function(name){
        var tagModel = {NAME: name};
        this._dirrectoryTags.push(tagModel);
        return tagModel;
    },
    getTagModel: function(name){
        return this._dirrectoryTags.find(function (item){return item.NAME === name});
    },
    getAreaModelOrig: function(number){
        var da, plus, prefix;
        if (!number)
            return null;
        if (number.indexOf("+")===0)
            plus = true;
        else
            plus = false;
        if (plus && number.length < 7)
            return null;
        else if (number.length < 6)
            return null;

        for(var i=0,l=this._dirrectoryAreas.length;i<l;i++){
            da = this._dirrectoryAreas[i];

            if ((plus && da['LENGTH'] > number.length - 1) || (!plus && da['LENGTH'] > number.length)) {
                continue;
            }

            if (plus) {
                prefix = number.substring(1, da['LENGTH'] + 1);
            } else {
                prefix = number.substring(0, da['LENGTH']);
            }

            if (prefix == da['NUMBER']) {
                return da;
            }
        }
        return null;
    },
    getAreaModel: function(number){
        var prefix;
        if (!number)
            return null;
        if (typeof number !== 'string')
            return ''+number;
        if (number.indexOf("+")===0)
            number = number.substring(1);
        if (number.length < 6)
            return null;
        for (var length=9;length>=1;length--){
            if (length > number.length) {
                continue;
            }
            prefix = number.substring(0, length);
            if (this._dirrectoryAreasMap[length]!==undefined && this._dirrectoryAreasMap[length][prefix]!==undefined){
                return this._dirrectoryAreas[this._dirrectoryAreasMap[length][prefix]];
            }
        }
        return null;
    },
    getTrunkModel: function(channel){
    		// if trunk name contain space need decode %20 to space
    	    channel = decodeURIComponent(channel);
        var dt, identy;

        for(var i=0,l=this._dirrectoryTruncs.length;i<l;i++){
            identy = "/" + this._dirrectoryTruncs[i]['NAME'];
            if (channel && channel.indexOf(identy)!==-1){
                return this._dirrectoryTruncs[i];
            }
        }
        return null;
    },
    getPbxModel: function(serial){
        return serial && this._dirrectoryPbxes.has(serial) ? this._dirrectoryPbxes.get(serial) :  null;
    },
    getHour: function (dateStr) {
        /* HANDS UP! loss offset between PBX timezone and UTC! */
        var seconds = Date.parse((dateStr + "Z" || '').replace(' ', 'T')) / 1000;

        return isNaN(seconds) ? 0 : parseInt((seconds - (seconds % 3600)) / 3600);
    },
    getDay: function(dateStr){
        //var seconds = Date.parse((dateStr||'').replace(' ','T'))/1000;

        if (navigator.userAgent.indexOf("Firefox") !== -1 && dateStr.indexOf(':') === -1) {
            dateStr = dateStr + ' 00:00';
        }

        var seconds = new Date(dateStr + "Z" || '').getTime() / 1000;
        var result = parseInt( (seconds - (seconds % 86400)) / 86400 ) || 0;

        if (!isNaN(seconds)) {
            return result;
        }
        else {
            return 0;
        }
    },
    getMonth: function(dateStr){
        // YEAR concatenation with month
        return parseInt(dateStr.substring(0, 4)+dateStr.substring(5, 7)) || 0;
    },
    getYear: function(dateStr){
        return parseInt(dateStr.substring(0, 4)) || 0;
    },
    getHourOfDay: function(date){
        return date.getHours();
    },
    getDayOfWeek: function(date){
        return date.getDay();
    },
    getMonthOfYear: function(date){
        return date.getMonth();
    },
    parseTags: function(data){
        var tags = [];
        if (data && data.indexOf('tags:')===0 ){
            var splits = data.substring(5).split(",");
            var tag;
            for (var i=0;i<splits.length;i++) {
                tag = splits[i].trim().toLowerCase();
                if (tag) {
                    tags.push(tag);
                }
            }
        }
        return tags;
    },
    getUserModel: function(number, isEmergencyCall){
        if (!number){
        	return null;
        }

        if (!isEmergencyCall && this._dirrectoryUsersMapLocalNUMBER.has(number)) {
            return this._dirrectoryUsersMapLocalNUMBER.get(number);
        }
        if (this._dirrectoryUsersMapRemoteNUMBER.has(number)) {
            return this._dirrectoryUsersMapRemoteNUMBER.get(number);
        }
        if (this._dirrectoryUsersMapMOBILE_NUMBER.has(number)) {
            return this._dirrectoryUsersMapMOBILE_NUMBER.get(number);
        }
        if (this._dirrectoryUsersMapFAX_NUMBER.has(number)) {
            return this._dirrectoryUsersMapFAX_NUMBER.get(number);
        }
        if (this._dirrectoryUsersMapOFFICE_NUMBER.has(number)) {
            return this._dirrectoryUsersMapOFFICE_NUMBER.get(number);
        }
        if (this._dirrectoryUsersMapHOME_NUMBER.has(number)) {
            return this._dirrectoryUsersMapHOME_NUMBER.get(number);
        }
        if (this._dirrectoryUsersMapHOME_MOBILE_NUMBER.has(number)) {
            return this._dirrectoryUsersMapHOME_MOBILE_NUMBER.get(number);
        }
        if (this._dirrectoryUsersMapPHONE_NUMBER.has(number)) {
            return this._dirrectoryUsersMapPHONE_NUMBER.get(number);
        }
        return null;
    },

    addUserModel: function(model){
        this._dirrectoryUsers.push(model);

        if (model['TYPE']==1){
            if (model['NUMBER']) {
                this._dirrectoryUsersMapLocalNUMBER.set(model['NUMBER'], model);
            }
        } else {
            if (model['NUMBER']) {
                this._dirrectoryUsersMapRemoteNUMBER.set(model['NUMBER'], model);
            }
        }
        if (model['MOBILE_NUMBER']) {
            this._dirrectoryUsersMapMOBILE_NUMBER.set(model['MOBILE_NUMBER'], model);
        }
        if (model['FAX_NUMBER']) {
            this._dirrectoryUsersMapFAX_NUMBER.set(model['FAX_NUMBER'], model);
        }
        if (model['OFFICE_NUMBER']) {
            this._dirrectoryUsersMapOFFICE_NUMBER.set(model['OFFICE_NUMBER'], model);
        }
        if (model['HOME_NUMBER']) {
            this._dirrectoryUsersMapHOME_NUMBER.set(model['HOME_NUMBER'], model);
        }
        if (model['HOME_MOBILE_NUMBER']) {
            this._dirrectoryUsersMapHOME_MOBILE_NUMBER.set(model['HOME_MOBILE_NUMBER'], model);
        }
        if (model['PHONE_NUMBER']) {
            this._dirrectoryUsersMapPHONE_NUMBER.set(model['PHONE_NUMBER'], model);
        }
    },

    setTrunksModel: function(models){
        this._dirrectoryTruncs = models;
    },

    setPbxesModel: function(models){
        var self = this;
        models.forEach(function(item) {
            self._dirrectoryPbxes.set(item.SERIAL, item);
        });
    },

    getDirrectories: function(){
        var self = this,
            storage = this.getStorage(),
            seq = Promise.resolve();
        if (!this._dirrectoryFetched){
            seq = seq
                .then(function(){
                    return storage.fetchPromise('SELECT \n\
                    ID,GROUP_ID,TYPE,NAME,NAME_LOWER,NUMBER,MOBILE_NUMBER,FAX_NUMBER,OFFICE_NUMBER,HOME_NUMBER,HOME_MOBILE_NUMBER,PHONE_NUMBER\n\
                    FROM `USERS` ORDER BY `TYPE` ASC')
                        .then(function(result){
                            for (var i=0;i<result.length;i++){
                                self.addUserModel(result[i]);
                            }
                        });
                })
                .then(function(){
                    return storage.fetchPromise('SELECT ID,NAME FROM `TRUNKS`')
                        .then(function(result){
                            self.setTrunksModel(result);
                        });
                })
                .then(function(){
                    return storage
                        .fetchPromise('SELECT ID,COUNTRY_ID,NUMBER,`LENGTH` FROM `COUNTRIES_AREAS` ORDER BY `LENGTH` DESC')
                        .then(function(result){
                            self._dirrectoryAreas = result;
                            for (var i=0;i<result.length;i++){
                                if (self._dirrectoryAreasMap[result[i]['LENGTH']] === undefined)
                                    self._dirrectoryAreasMap[result[i]['LENGTH']] = {};
                                if (self._dirrectoryAreasMap[result[i]['LENGTH']][result[i]['NUMBER']] === undefined)
                                    self._dirrectoryAreasMap[result[i]['LENGTH']][result[i]['NUMBER']] = i;
                            }
                        });
                })
                .then(function(){
                    return storage.fetchPromise('SELECT ID,NAME FROM `TAGS`').then(function(result){
                        self._dirrectoryTags = result;
                    });
                })
                .then(function(){
                    return storage.fetchPromise('SELECT ID,SERIAL FROM `PBXES`').then(function(result){
                        self.setPbxesModel(result);
                    });
            });
        }

        seq = seq.then(function(){
            self._dirrectoryFetched = true;
            return;
        });
        return seq;
    },
    getUserModelByNumber: function(number, isEmergencyCall) {
        number = number.replace('+', '');

        // trying find user with number with a plus
        var user = this.getUserModel('+' + number, isEmergencyCall);

        if (!user) {
            // trying find user with number without a plus
            user = this.getUserModel(number, isEmergencyCall);
        }

        return user;
    },
    addExternalUserModel: function (user) {
        this.addUserModel({
            ID: user.id,
            GROUP_ID: user.groupId,
            TYPE: user.type,
            NAME: user.name,
            NAME_LOWER: user.name.toLowerCase(),
            NUMBER: user.number,
            MOBILE_NUMBER: user.mobile_number,
            FAX_NUMBER: user.fax_number,
            OFFICE_NUMBER: user.office,
            HOME_NUMBER: user.home,
            HOME_MOBILE_NUMBER: user.home_mobile,
            PHONE_NUMBER: user.phone
        });
    },
    generateUserData: function (number, name) {
        if(!this._userIds.size) {
            for (var i =0; i < this._dirrectoryUsers.length; i++) {
                this._userIds.add(this._dirrectoryUsers[i]['ID']);
            }
        }

        var id = this.generateId();

        while (this._userIds.has(id)) {
            id = this.generateId();
        }

        this._userIds.add(id);

        return {
            id: id,
            name: name,
            phone: number,
            number: '',
            fax_number: '',
            groupId: null,
            home: '',
            home_mobile: '',
            mobile_number: '',
            office: '',
            organization: '',
            type: 2 // external user
        }
    },
    generateId: function () {
        var min = 100000000000;
        var max = 900000000000;
        return Math.floor(Math.random() * (max - min + 1) + min);
    }
});
