Ext.define('App.proxy.Groups', {
    extend: 'App.proxy.AbstractProxy',
    alias: 'proxy.groups',
    constructor	: function(config) {
		this.callParent(arguments);
    },
    pushGroupsPromises: function(operations){
        var self = this,
            esc = this.escape,
            storage = this.getStorage(),
            seq = Promise.resolve();
        var inserts = [],updatesIgnore = [],updatesDisIgnore = [],
            deletes = [],updateGroupUsers = [],insertWthUserNumbers = [];
        for (var i=0; i<operations.length; i++){
            switch (operations[i].type){
                case 'insert': inserts.push(operations[i].data);break;
                case 'ignore': updatesIgnore.push(operations[i].data);break;
                case 'disIgnore': updatesDisIgnore.push(operations[i].data);break;
                case 'delete': deletes.push(operations[i].data);break;
                case 'updateGroupUsers': updateGroupUsers.push(operations[i].data);break;
                case 'insertWthUserNumbers': insertWthUserNumbers.push(operations[i].data);break;
            }
        }

        if (inserts.length>0){
            var insertRows = [];
            for (var i=0; i < inserts.length; i++) {
                var d = inserts[i];
                insertRows.push({
                        ID: null,
                        UID: (d['UID']||'-'),
                        NAME: d['NAME'],
                        NAME_LOWER: d['NAME_LOWER'],
                        TYPE: d['TYPE'],
                        IGNORE: d['IGNORE']
                });
            }
            seq = seq.then(function(){
                return storage.bulkInsertPromise('GROUPS',insertRows);
            });
        }
        if (updatesIgnore.length>0){
            var ids = [];
            for (var i=0; i < updatesIgnore.length; i++) {
                if (Ext.Array.indexOf(ids,updatesIgnore[i]['ID'])==-1)
                    ids.push( updatesIgnore[i]['ID'] );
            }
            var updateIgnoreSql = 'UPDATE GROUPS SET `IGNORE` = 1 WHERE ID IN ('+ids.join(',')+')';
            var updateUsersSql = 'UPDATE `USERS` SET `GROUP_ID` = NULL WHERE `GROUP_ID` IN ( '+ids.join(',')+')';
            //
            seq = seq.then(function(){
                return storage.executePromise(updateIgnoreSql).then(function(){
                    return storage.executePromise(updateUsersSql);
                });
            });
        }
        if (updatesDisIgnore.length>0){
            var ids = [];
            for (var i=0; i < updatesDisIgnore.length; i++) {
                if (Ext.Array.indexOf(ids,updatesDisIgnore[i]['ID'])==-1)
                    ids.push( updatesDisIgnore[i]['ID'] );
            }
            var updateDisIgnoreSql = 'UPDATE GROUPS SET `IGNORE` = 0 WHERE ID IN ( '+ids.join(',')+')';
            //
            seq = seq.then(function(){
                return storage.executePromise(updateDisIgnoreSql);
            });
        }
        if (deletes.length>0){
            var ids = [];
            for (var i=0; i < deletes.length; i++) {
                if (Ext.Array.indexOf(ids,deletes[i]['ID'])==-1)
                    ids.push( deletes[i]['ID'] );
            }
            var deleteSql = 'DELETE FROM GROUPS WHERE ID IN ( '+ids.join(',')+')';
            //
            seq = seq.then(function(){
                return storage.executePromise(deleteSql);
            });
        }
        if (updateGroupUsers.length>0){
            console.debug('updateGroupUsers',JSON.stringify(updateGroupUsers));
            var inUsersNumbers,groupId;
            for (var i=0; i < updateGroupUsers.length; i++) {
                inUsersNumbers = updateGroupUsers[i]['numbers'].reduce(function(numbers,n){
                    if (n)
                        return  (numbers? numbers + ',':'')+esc(n);
                    else
                        return  numbers;
                },'');
                groupId = esc(updateGroupUsers[i]['ID']);

                (function(){
                    var updateOldUsersSql = "UPDATE `USERS` SET `GROUP_ID` = NULL WHERE `GROUP_ID` = "+groupId;
                    var updateNewUsersSql = "UPDATE `USERS` SET `GROUP_ID` = "+groupId+" WHERE `NUMBER` IN ("+inUsersNumbers+")";
                    seq = seq.then(function(){
                        return storage.executePromise(updateOldUsersSql).then(function(){
                            return storage.executePromise(updateNewUsersSql);
                        });
                    });
                })();

            }
        }
        if (insertWthUserNumbers.length>0){
            for (var i=0; i < insertWthUserNumbers.length; i++) {
                (function(){
                    var inUsersNumbers = insertWthUserNumbers[i]['numbers'].reduce(function(numbers,n){
                        if (n)
                            return  (numbers? numbers + ',':'')+esc(n);
                        else
                            return  numbers;
                    },'');
                    var d = insertWthUserNumbers[i];
                    seq = seq.then(function(){
                        return storage.bulkInsertPromise('GROUPS',[{
                                ID: null,
                                UID: (d['UID']||'-'),
                                NAME: d['NAME'],
                                NAME_LOWER: d['NAME_LOWER'],
                                TYPE: d['TYPE'],
                                IGNORE: d['IGNORE']
                        }]).then(function(response){
                            var groupId = esc(response.lastInsertId);
                            var updateNewUsersSql = "UPDATE `USERS` SET `GROUP_ID` = "+groupId+" WHERE `NUMBER` IN ("+inUsersNumbers+")";
                            return storage.executePromise(updateNewUsersSql);
                        });
                    });
                })();
            }
        }

        seq = seq.then(function(){
            console.debug('Seq done!');
            if (inserts.length || updatesIgnore.length || updatesDisIgnore.length ||
                deletes.length || updateGroupUsers.length || insertWthUserNumbers.length ){
                storage.fetchPromise('SELECT ID,UID,NAME,NAME_LOWER,TYPE,`IGNORE` FROM GROUPS').
                        then(function(availableGroups){
                            self._dbGroups = availableGroups;
                        });

            }
            return;
        });
        return seq;
    },
    _dbGroups:null,
    pushGroups: function(items, callback){
        var self = this,
            esc = self.escape,
            storage = self.getStorage();
console.debug('pushGroups items', items);
        if (items.length==0)
            callback();
        Promise.resolve().then(function(){
                if (self._dbGroups == null)
                    return storage.fetchPromise('SELECT ID,UID,NAME,NAME_LOWER,TYPE,`IGNORE` FROM GROUPS');
                else
                    return self._dbGroups;
            }).
            then(function(availableGroups){
                self._dbGroups = availableGroups;
                var operations = [];
                var numbers = [];
                var availableGroupInDb;
                for (var i=0; i < items.length; i++) {
                    availableGroupInDb = false;
                    for (var c=0; c < availableGroups.length; c++) {
                        if (availableGroups[c]['TYPE'] == 1 && availableGroups[c]['UID'] ==  items[i][0]) {
                            availableGroupInDb = availableGroups[c];
                            break;
                        }
                    }
                    numbers = items[i][2];
                    if (availableGroupInDb){
                        //@todo check - in old realisation in applet compared group.getGroupIgnore() != 1 Why?
                        if (availableGroupInDb['IGNORE'] == 1) {
                            operations.push({type:'disIgnore',data:{
                                    ID:availableGroupInDb['ID']
                            }});
                        }

                        operations.push({type:'updateGroupUsers',data:{
                                ID:availableGroupInDb['ID'],numbers:numbers
                        }});
                    } else {
                        operations.push({type:'insertWthUserNumbers',data:{
                                UID:items[i][0],
                                NAME:items[i][1],
                                NAME_LOWER:(items[i][1]||'').toLowerCase(),
                                TYPE:1,
                                IGNORE:0,
                                numbers:numbers
                        }});

                    }
                }
                for (var i=0; i < availableGroups.length; i++) {
                    availableGroupInDb = false;
                    for (c=0; c < items.length; c++) {
                        if (availableGroups[i]['TYPE'] == 1 && availableGroups[i]['UID'] ==  items[c][0]) {
                            availableGroupInDb = true;
                        }
                    }
                    if (availableGroups[i]['TYPE'] == 1 && availableGroups[i]['IGNORE'] == 0 && !availableGroupInDb) {
                        operations.push({type:'ignore',data:{
                            ID:availableGroups[i]['ID']
                        }});
                    }
                }
                return self.pushGroupsPromises(operations);
            }).then(callback)
        ['catch'](function(e){console.debug('pushGroups error', e , arguments);});

    }
});
