Ext.define('App.proxy.Pbxes', {
    extend: 'App.proxy.AbstractProxy',
    alias: 'proxy.pbxes',
    read: function(operation, callback, scope) {
        var sql = "SELECT `ID`, `SERIAL`, `NAME` FROM `PBXES` ORDER BY `SERIAL` ASC";
        var controller = function(records) {
            var i, result, pbxes, reader;

            pbxes = new Array();

            for( i=0; i < records.length; i++ ) {
                pbxes.push( { id: records[i]['ID'], serial: records[i]['SERIAL'], name: records[i]['NAME'] } );
            }

            reader = this.getReader();
            result = reader.read(pbxes);

            operation.setCompleted();
            operation.setSuccessful();

            Ext.apply(operation, { resultSet: result } );
            Ext.callback(callback, scope || this, [operation]);
        }

        this.getStorage().batch(
            [App.storage.Storage.METHOD_FETCH, sql],
            controller,
            this
        );
    },
    readCount: function(callback, scope) {
        scope = scope || this;

        var sql = "SELECT COUNT(*) FROM `PBXES`";
        var controller = function() {
            callback.apply(scope, arguments);
        }

        this.getStorage().fetchOneAsync(sql, controller);
    },
    pushPbxesPromises: function(operations){
        var self = this,
            esc = this.escape,
            storage = this.getStorage(),
            seq = Promise.resolve();
        var inserts = [],updates = [],updatesIgnore = [],updatesDisIgnore = [];
        for (var i=0; i<operations.length; i++){
            switch (operations[i].type){
                case 'insert': inserts.push(operations[i].data);break;
                case 'update': updates.push(operations[i].data);break;
                case 'ignore': updatesIgnore.push(operations[i].data);break;
                case 'disIgnore': updatesDisIgnore.push(operations[i].data);break;
            }
        }
        if (inserts.length>0){
            var insertSql = 'INSERT INTO PBXES(ID,SERIAL,NAME,NAME_LOWER,`IGNORE`) \n\
                             VALUES ';
            for (var i=0; i < inserts.length; i++) {
                var d = inserts[i];
                insertSql += '(null,'+ esc(d['SERIAL'])+','+ esc(d['NAME'])+','+esc(d['NAME_LOWER'])+','+esc(d['IGNORE'])+') , ';
            }
            insertSql = insertSql.substr(0,insertSql.length-2) + '; ';
            //
            seq = seq.then(function(){
                return storage.executePromise(insertSql);
            });
        }
        if (updates.length>0){
            for (var i=0; i < updates.length; i++) {
                var d = updates[i];
                var updateSql = 'UPDATE PBXES SET NAME = '+ esc(d['NAME'])+', '
                                +' NAME_LOWER = '+esc(d['NAME_LOWER'])
                                +' WHERE ID = '+d['ID'];
                //
                seq = seq.then(Ext.bind(function(updateSql){
                    return storage.executePromise(updateSql);
                },storage,[updateSql]));
            }
        }
        if (updatesIgnore.length>0){
            var ids = [];
            for (var i=0; i < updatesIgnore.length; i++) {
                if (Ext.Array.indexOf(ids,updatesIgnore[i]['ID'])==-1)
                    ids.push( updatesIgnore[i]['ID'] );
            }
            var updateIgnoreSql = 'UPDATE PBXES SET `IGNORE` = 1 WHERE ID IN ('+ids.join(',')+')';
            //
            seq = seq.then(function(){
                return storage.executePromise(updateIgnoreSql);
            });
        }
        if (updatesDisIgnore.length>0){
            var ids = [];
            for (var i=0; i < updatesDisIgnore.length; i++) {
                if (Ext.Array.indexOf(ids,updatesDisIgnore[i]['ID'])==-1)
                    ids.push( updatesDisIgnore[i]['ID'] );
            }
            var updateDisIgnoreSql = 'UPDATE PBXES SET `IGNORE` = 0 WHERE ID IN ( '+ids.join(',')+')';
            //
            seq = seq.then(function(){
                return storage.executePromise(updateDisIgnoreSql);
            });
        }

        seq = seq.then(function(){
            console.debug('Seq Pbxe done!');
            if (inserts.length>0 || updatesIgnore.length>0 || updatesDisIgnore.length>0){
                return storage.fetchPromise('SELECT ID,SERIAL,NAME,NAME_LOWER,`IGNORE` FROM PBXES').
                        then(function(availablePbxes){
                            storage.proxyEvents.setPbxesModel(availablePbxes);
                        });
            }
            return;
        });
        return seq;
    },
    pushPbxes: function(items, callback){
        var self = this,
            esc = self.escape,
            storage = self.getStorage();
        function getPbxName(serial, domain){
            if (serial && domain)
                return domain+' ('+serial+')';
            if (domain)
                return domain;
            return serial;
        }
console.debug('pushPbxes items', items);
        if (items.length==0)
            callback();
        storage.fetchPromise('SELECT ID,SERIAL,NAME,NAME_LOWER,`IGNORE` FROM PBXES').
            then(function(availablePbxes){
                var operations = [];
                var numbers = [];
                var availablePbxeInDb;
                for (var i=0; i < items.length; i++) {
                    availablePbxeInDb = false;
                    for (var c=0; c < availablePbxes.length; c++) {
                        if (availablePbxes[c]['SERIAL'] ==  items[i][0]) {
                            availablePbxeInDb = availablePbxes[c];
                            break;
                        }
                    }

                    if (availablePbxeInDb){
                        if (availablePbxeInDb['IGNORE'] == 1) {
                            operations.push({type:'disIgnore',data:{
                                    ID:availablePbxeInDb['ID']
                            }});
                        }
                        if (availablePbxeInDb['NAME'] != getPbxName(items[i][0], items[i][1])) {
                            operations.push({type:'update',data:{
                                    ID:availablePbxeInDb['ID'],
                                    NAME:getPbxName(items[i][0], items[i][1]),
                                    NAME_LOWER:(items[i][1]||'').toLowerCase()
                            }});
                        }
                    } else {
                        operations.push({type:'insert',data:{
                                SERIAL:items[i][0],
                                NAME:getPbxName(items[i][0], items[i][1]),
                                NAME_LOWER:(items[i][1]||'').toLowerCase(),
                                IGNORE:0
                        }});

                    }
                }
                for (var i=0; i < availablePbxes.length; i++) {
                    availablePbxeInDb = false;
                    for (c=0; c < items.length; c++) {
                        if (availablePbxes[i]['SERIAL'] ==  items[c][0]) {
                            availablePbxeInDb = true;
                        }
                    }
                    if (availablePbxes[i]['IGNORE'] == 0 && !availablePbxeInDb) {
                        operations.push({type:'ignore',data:{
                            ID:availablePbxes[i]['ID']
                        }});
                    }
                }
                return self.pushPbxesPromises(operations);
            }).then(callback)
        ['catch'](function(e){console.debug('pushPbxes error', e , arguments);});

    }
});
