Ext.define('App.proxy.Trunks', {
    extend: 'App.proxy.AbstractProxy',
    alias: 'proxy.trunks',
    read: function(operation, callback, scope) {
        var sql = "SELECT * FROM `TRUNKS` ORDER BY `NAME` ASC";
        var controller = function(records) {
            var i, result, trunks, reader;

            trunks = new Array();

            for( i=0; i < records.length; i++ ) {
                trunks.push( { id: records[i]['ID'], name: records[i]['NAME'] } );
            }

            reader = this.getReader();
            result = reader.read(trunks);

            operation.setCompleted();
            operation.setSuccessful();

            Ext.apply(operation, { resultSet: result } );
            Ext.callback(callback, scope || this, [operation]);
        }

        this.getStorage().batch(
            [App.storage.Storage.METHOD_FETCH, sql],
            controller,
            this
        );
    },
    readCount: function(callback, scope) {
        scope = scope || this;
        if (this._dbTrunks != null)
            return callback.apply(scope, [this._dbTrunks.length]);
        var sql = "SELECT COUNT(*) FROM `TRUNKS`";
        var controller = function() {
            callback.apply(scope, arguments);
        }

        this.getStorage().fetchOneAsync(sql, controller);
    },
    pushTrunksPromises: function(operations){
        var self = this,
            esc = this.escape,
            storage = this.getStorage(),
            seq = Promise.resolve();
        var inserts = [],updatesIgnore = [],updatesDisIgnore = [];
        for (var i=0; i<operations.length; i++){
            switch (operations[i].type){
                case 'insert': inserts.push(operations[i].data);break;
                case 'ignore': updatesIgnore.push(operations[i].data);break;
                case 'disIgnore': updatesDisIgnore.push(operations[i].data);break;
            }
        }
        if (inserts.length>0){
            var insertSql = 'INSERT INTO TRUNKS(ID,NAME,NAME_LOWER,`IGNORE`) \n\
                             VALUES ';
            for (var i=0; i < inserts.length; i++) {
                var d = inserts[i];
                insertSql += '(null,'+ esc(d['NAME'])+','+esc(d['NAME_LOWER'])+','+esc(d['IGNORE'])+') , ';
            }
            insertSql = insertSql.substr(0,insertSql.length-2) + '; ';
            //
            seq = seq.then(function(){
                return storage.executePromise(insertSql);
            });
        }
        if (updatesIgnore.length>0){
            var ids = [];
            for (var i=0; i < updatesIgnore.length; i++) {
                if (Ext.Array.indexOf(ids,updatesIgnore[i]['ID'])==-1)
                    ids.push( updatesIgnore[i]['ID'] );
            }
            var updateIgnoreSql = 'UPDATE TRUNKS SET `IGNORE` = 1 WHERE ID IN ('+ids.join(',')+')';
            //
            seq = seq.then(function(){
                return storage.executePromise(updateIgnoreSql);
            });
        }
        if (updatesDisIgnore.length>0){
            var ids = [];
            for (var i=0; i < updatesDisIgnore.length; i++) {
                if (Ext.Array.indexOf(ids,updatesDisIgnore[i]['ID'])==-1)
                    ids.push( updatesDisIgnore[i]['ID'] );
            }
            var updateDisIgnoreSql = 'UPDATE TRUNKS SET `IGNORE` = 0 WHERE ID IN ( '+ids.join(',')+')';
            //
            seq = seq.then(function(){
                return storage.executePromise(updateDisIgnoreSql);
            });
        }

        seq = seq.then(function(){
            console.debug('Seq Trunk done!');
            if (inserts.length>0 || updatesIgnore.length>0 || updatesDisIgnore.length>0){
                return storage.fetchPromise('SELECT ID,NAME,NAME_LOWER,`IGNORE` FROM TRUNKS').
                        then(function(availableTrunks){
                            self._dbTrunks = availableTrunks;
                            storage.proxyEvents.setTrunksModel(availableTrunks);
                        });
            }
            return;
        });
        return seq;
    },
    _dbTrunks: null,
    pushTrunks: function(items, callback){
        var self = this,
            esc = self.escape,
            storage = self.getStorage();
console.debug('pushTrunks items', items);
        if (items.length==0)
            callback();
        Promise.resolve().then(function(){
                if (self._dbTrunks == null)
                    return storage.fetchPromise('SELECT ID,NAME,NAME_LOWER,`IGNORE` FROM TRUNKS');
                else
                    return self._dbTrunks;
            }).
            then(function(availableTrunks){
                self._dbTrunks = availableTrunks;
                var operations = [];
                var numbers = [];
                var availableTrunkInDb;
                for (var i=0; i < items.length; i++) {
                    availableTrunkInDb = false;
                    for (var c=0; c < availableTrunks.length; c++) {
                        if (availableTrunks[c]['NAME'] ==  items[i]) {
                            availableTrunkInDb = availableTrunks[c];
                            break;
                        }
                    }

                    if (availableTrunkInDb){
                        //@todo check - in old realisation in applet compared getTrunkIgnore() != 1 Why?
                        if (availableTrunkInDb['IGNORE'] == 1) {
                            operations.push({type:'disIgnore',data:{
                                    ID:availableTrunkInDb['ID']
                            }});
                        }
                    } else {
                        operations.push({type:'insert',data:{
                                NAME:items[i],
                                NAME_LOWER:(items[i]||'').toLowerCase(),
                                IGNORE:0
                        }});

                    }
                }
                for (var i=0; i < availableTrunks.length; i++) {
                    availableTrunkInDb = false;
                    for (c=0; c < items.length; c++) {
                        if (availableTrunks[i]['NAME'] ==  items[c]) {
                            availableTrunkInDb = true;
                        }
                    }
                    if (availableTrunks[i]['IGNORE'] == 0 && !availableTrunkInDb) {
                        operations.push({type:'ignore',data:{
                            ID:availableTrunks[i]['ID']
                        }});
                    }
                }
                return self.pushTrunksPromises(operations);
            }).then(callback)
        ['catch'](function(e){console.debug('pushTrunks error', e , arguments);});

    }
});
