Ext.define('App.view.filter.TrunksEditor', {
    extend: 'App.view.filter.Editor',
    width: 400,
    onRenderContent: function(state) {
        this.trunksField = Ext.create('App.view.filter.field.Trunk', {
            fieldLabel: _t('Trunks'),
            labelAlign: 'left',
            labelWidth: 100,
            arrayValue: state['trunks'],
            anchor: '100%'
        });
        
        this.containerEl = Ext.create('Ext.container.Container', {
            renderTo: this.formEl,
            layout: 'anchor',
            items: [
                this.trunksField
            ]
        });
        
        this.trunksField.on('change', this.onTrunksFieldChange, this);
        this.callParent(arguments);
    },
    onTrunksFieldChange: function(field) {
        if (this.ignoreStateEvents) {
            return;
        }
        
        this.state['trunks'] = field.getArrayValue();
        this.fireEvent('change', this.state);
    },
    hide: function(){
        if(this.trunksField.selectedView != undefined && this.trunksField.selectedView.isVisible()) {
            this.trunksField.hidePicker();
            return;
        } else {
            this.callParent();
        }
    },
    setState: function(s) {
        this.state = s;
        this.conditionEl.update(this.getDescription(s));
        
        if (!this.contentRendered) {
            return;
        }
        
        this.ignoreStateEvents = true;
        this.trunksField.setArrayValue(s['trunks']);
        this.ignoreStateEvents = false;
    },
    getDefaultState: function() {
        var s = {
            'trunks': new Array()
        }
        
        return s;
    },
    
    getTrunksDescription: function(trunks) {
        var i, result = new Array();
        
        if( trunks.length == 0 ) {
            return undefined;
        }
        
        for( i=0; i < trunks.length; i++ ) {
            result.push(trunks[i]['name']);
        }
        
        if ( result.length > 0 ) {
            return result.join(', ');
        } else {
            return undefined;
        }      
    },
    
    getDescription: function(state) {
        var desc = this.getTrunksDescription(state['trunks']);
        
        if (desc) {
            return sprintf('%(trunkDesc)s', { trunkDesc : desc });
        }
        
        return this.getAnyDescription();
    },
    
    getFieldDescription: function(state) {
        var desc = this.getTrunksDescription(state['trunks']);
        
        if (desc) {
            return sprintf('%(trunkDesc)s', { trunkDesc : desc });
        }
        
        return this.getAnyFieldDescription();
    },
    
    getFilterDescription: function() {
        var desc = this.getTrunksDescription(this.state['trunks']);
        
        if (desc) {
            return _t('Trunks')+': '+ desc;
        }
        
        return null;
    }
});
