Ext.define('App.view.filter.TypesEditor', {
    extend: 'App.view.filter.Editor',
    width: 400,
    onRenderContent: function(state) {
        this.checkboxCallEl = Ext.create('Ext.form.field.Checkbox', {
            boxLabel: _t('Call'),
            checked: state['call']
        });
        this.checkboxVoicemailEl = Ext.create('Ext.form.field.Checkbox', {
            boxLabel: _t('Voicemail'),
            checked: state['voicemail']
        });
        this.checkboxFaxEl = Ext.create('Ext.form.field.Checkbox', {
            boxLabel: _t('Fax'),
            checked: state['fax']
        });
         
       this.checkboxGroup = Ext.create('Ext.form.CheckboxGroup', {
            renderTo: this.formEl,
            fieldLabel: _t('Services'),
            columns: 2,
            width: 376,
            vertical: true,
            margin: '0 0 -10',
            items: [
                this.checkboxCallEl,
                this.checkboxVoicemailEl,
                this.checkboxFaxEl
            ]
        });

        this.checkboxCallEl.on('change', this.onCheckboxCallElChange, this);
        this.checkboxVoicemailEl.on('change', this.onCheckboxVoicemailElChange, this);
        this.checkboxFaxEl.on('change', this.onCheckboxFaxElChange, this);
        this.callParent(arguments);
    },
    onCheckboxCallElChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }
        
        this.state['call'] = value;
        this.fireEvent('change', this.state);
    },
    onCheckboxVoicemailElChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }
        
        this.state['voicemail'] = value;
        this.fireEvent('change', this.state);
    },
    onCheckboxFaxElChange: function(field, value) {
        if (this.ignoreStateEvents) {
            return;
        }
        
        this.state['fax'] = value;
        this.fireEvent('change', this.state);
    },
    setState: function(s) {
        this.state = s;
        this.conditionEl.update(this.getDescription(s));
        
        if (!this.contentRendered) {
            return;
        }
        
        this.ignoreStateEvents = true;
        this.checkboxCallEl.setValue(this.state['call']);
        this.checkboxVoicemailEl.setValue(this.state['voicemail']);
        this.checkboxFaxEl.setValue(this.state['fax']);
        this.ignoreStateEvents = false;
    },
    getDefaultState: function() {
        var s = new Object();
            s['call'] = false;
            s['voicemail'] = false;
            s['fax'] = false;

        return s;
    },
    
    /**
     * Get's a selected types.
     */
    getTypesDescription: function(state) {
        var types = new Array();
        
        if (state['call']) {
            types.push(_t('Calls'));
        }
        if (state['voicemail']) {
            types.push(_t('Voicemail'));
        }
        if (state['fax']) {
            types.push(_t('Faxes'));
        }
        
        return types;
    },
    
    getDescription: function(state) {
        var types = this.getTypesDescription(state);
       
        if (state['call'] && state['voicemail'] && state['fax']) {
            return this.getAnyDescription();
        }
        if (types.length > 0) {
            return sprintf("%(type)s", { type : types.join(', ') });
        }
        
        return this.getAnyDescription();
    },
    
    getFieldDescription: function(state) {
        var types = this.getTypesDescription(state);
        
        if (state['call'] && state['voicemail'] && state['fax']) {
            return this.getAnyFieldDescription();
        }
        if (types.length > 0) {
            return sprintf("%(type)s", { type : types.join(', ') });
        }
        
        return this.getAnyFieldDescription();
    },
    
    getFilterDescription: function() {
        var s = this.state;
        var types = this.getTypesDescription(s);
        
        if (s['call'] && s['voicemail'] && s['fax']) {
            return null;
        }
        if (types.length > 0) {
            return _t('Services')+': '+ types.join(', ');
        }
        
        return null;
    }
} );
