Ext.define('App.view.navigation.Settings', {
    extend: 'Ext.window.Window',
    width: 300,
    title: _t('Settings'),
    modal: true,
    closable: false,
    resizable: false,
    closeAction: 'hide',
    bodyStyle: 'background: transparent; border: 0px;',
    padding: '5 10',
    period: App.utility.Date.LAST_2_WEEKS,
    slaPeriod: 20,
    missCallSecPeriod: 0,
    gradeOfAccuracy: 2,
    constructor: function() {
        this.addEvents('submit', 'cancel');
        this.callParent(arguments);
    },
    initComponent: function() {
        //Check period attr
        this.period = parseInt(this.period);
        this.slaPeriod = parseInt(this.slaPeriod);
        this.missCallSecPeriod = parseInt(this.missCallSecPeriod);
        this.gradeOfAccuracy = parseInt(this.gradeOfAccuracy);
        if (Ext.Array.indexOf(
                [
                    App.utility.Date.LAST_2_WEEKS, App.utility.Date.MONTH,
                    App.utility.Date.LAST_3_MONTH, App.utility.Date.LAST_6_MONTH,
                    App.utility.Date.LAST_12_MONTH
                ],
                this.period
            ) == -1 ) {
            this.period = App.utility.Date.LAST_2_WEEKS;
        }
        if (Ext.Array.indexOf([10,20,30,60,90,120],this.slaPeriod) == -1 ) {
            this.slaPeriod = 20;
        }
        if (this.missCallSecPeriod < 0 || this.missCallSecPeriod > 60 ) {
            this.missCallSecPeriod = 0;
        }
        this.descriptionCt = Ext.create('Ext.container.Container', {
            html: _t('Period')
        });

        this.periodsStore = Ext.create('Ext.data.Store', {
            fields: ['value', 'name'],
            data : [
                { value: App.utility.Date.LAST_2_WEEKS, name: _t('Last 2 weeks') },
                { value: App.utility.Date.MONTH, name: _t('Last month') },
                { value: App.utility.Date.LAST_3_MONTH, name: _t('Last 3 month') },
                { value: App.utility.Date.LAST_6_MONTH, name: _t('Last 6 month') },
                { value: App.utility.Date.LAST_12_MONTH, name: _t('Last 12 month') }
            ]
        });

        this.typesField = Ext.create('Ext.form.ComboBox', {
            store: this.periodsStore,
            queryMode: 'local',
            displayField: 'name',
            valueField: 'value',
            editable: false,
            margin: '10 0 10 0',
            value: this.period
        });

        this.slaDescriptionCt = Ext.create('Ext.container.Container', {
            html: _t('SLA, sec:')
        });

        this.slaTypesField = Ext.create('Ext.form.ComboBox', {
            store: Ext.create('Ext.data.Store', {
                fields: ['value', 'name'],
                data : [
                    { value: 10, name: "10" },
                    { value: 20, name: "20" },
                    { value: 30, name: "30" },
                    { value: 60, name: "60" },
                    { value: 90, name: "90" },
                    { value: 120, name: "120" }
                ]
            }),
            queryMode: 'local',
            displayField: 'name',
            valueField: 'value',
            editable: false,
            margin: '10 0 10 0',
            value: this.slaPeriod
        });

        this.missCallSecPeriodDescriptionCt = Ext.create('Ext.container.Container', {
            html: _t('Missed calls min seconds:')
        });
        this.missCallSecPeriodField = Ext.create('Ext.form.Number', {
            editable: true,
            allowDecimals: false,
            margin: '10 0 10 0',
            value: this.missCallSecPeriod,
            maxValue: 60,
            minValue: 0
        });

        this.gradeOfAccuracyDescriptionCt = Ext.create('Ext.container.Container', {
            html: _t('Decimal places number in Cost')
        });
        this.gradeOfAccuracyField = Ext.create('Ext.form.Number', {
            editable: true,
            allowDecimals: false,
            margin: '10 0 0 0',
            value: this.gradeOfAccuracy,
            maxValue: 4,
            minValue: 1
        });

        this.formCmp = Ext.create('Ext.form.Panel', {
            labelAlign: 'top',
            layout: 'anchor',
            defaults: {
                anchor: '100%'
            },
            border: false,
            style: 'background: transparent;',
            bodyStyle: 'background: transparent;',
            items: [
                this.descriptionCt,
                this.typesField,
                this.slaDescriptionCt,
                this.slaTypesField,
                this.missCallSecPeriodDescriptionCt,
                this.missCallSecPeriodField,
                this.gradeOfAccuracyDescriptionCt,
                this.gradeOfAccuracyField
            ]
        });


        this.submitButton = Ext.create('Ext.button.Button', {
            text: _t('Save')
        });

        this.cancelButton = Ext.create('Ext.button.Button', {
            text: _t('Cancel')
        });

        //Add elements
        this.items = [
            this.formCmp
        ];

        this.buttons = [
            this.submitButton,
            this.cancelButton
        ];

        this.submitButton.on('click', this.onSubmitButtonClick, this);
        this.cancelButton.on('click', this.onCancelButtonClick, this);
        this.on('show', this.onSettingsWindowShow, this);
        this.on('hide', this.onSettingsWindowHide, this);

        this.callParent(arguments);
    },

    onSettingsWindowShow: function() {
        if(!this.KeyMapOnEsc) {
            this.KeyMapOnEsc = new Ext.util.KeyMap(Ext.getBody(), [{
               key: Ext.EventObject.ESC,
               defaultEventAction: 'preventDefault',
               scope: this,
               fn: this.onCancelButtonClick
            }]);
        } else {
            this.KeyMapOnEsc.enable();
        }
    },

    onSettingsWindowHide: function() {
        this.KeyMapOnEsc.disable();
    },

    close: function() {
        // Verifies that mask has been removed.
        this.getEl().unmask();

        this.callParent(arguments);
    },

    /**
     * Fires when user clicks to a Submit button, we should invoke an submit event and show loading mask.
     */
    onSubmitButtonClick: function() {
        this.period = this.typesField.getValue();
        this.slaPeriod = this.slaTypesField.getValue();

        //Prevent wrong values [0..60]
        if(this.missCallSecPeriodField.getValue()==null)
            this.missCallSecPeriodField.setValue(0);
        if (this.missCallSecPeriodField.getValue()>60)
            this.missCallSecPeriodField.setValue(60);

        this.missCallSecPeriod = this.missCallSecPeriodField.getValue();
        this.gradeOfAccuracy = this.gradeOfAccuracyField.getValue();

        this.getEl().mask(_t('Please wait...')+' '+_t('Saving...'));
        this.fireEvent('submit', this.typesField.getValue(), this.slaTypesField.getValue(), this.missCallSecPeriodField.getValue(), this.gradeOfAccuracyField.getValue());
    },

    /**
     * Fires when user clicks to a Cancel button, we should just close a window.
     */
    onCancelButtonClick: function() {
        this.typesField.setValue(this.period);
        this.close();
    }
});
