Ext.define('App.view.reports.picker.Selection', {
    extend: 'Ext.view.View',
    itemId: 'loadReportSelection',
    floating: true,
    hidden: true,
    hideMode: 'display',
    deferEmptyText: true,
    cls: 'w-reports-picker-selection',
    constructor: function() {
        this.addEvents('selectReport', 'deleteReport', 'showSharedReports', 'showPersonalReports');
        this.on('showSharedReports', this.onShowSharedReports);
        this.on('showPersonalReports', this.onShowPersonalReports);
        this.callParent(arguments);
    },
    initComponent: function() {
        this.store = Ext.create('Ext.data.JsonStore', {
            fields: [
                { name:'id' },
                { name:'name' },
                { name:'description' },
                { name:'state' },
                { name:'step' },
                { name: 'deletable', defaultValue: false }
            ]
        });
        
        this.tpl = new Ext.XTemplate(
            '<div class="w-reports-picker-selection-container">',
            '<tpl for=".">',
                '<div class="item <tpl if="step == 1">item-even</tpl>">',
                    '<div class="item-name">{name}</div>',
                    '<div class="item-description">{description}</div>',
                    '{[this.getDeleteLink(values)]}',
                '</div>',
            '</tpl>',
            '</div>',
            {
                getDeleteLink: function(values) {
                    return values.deletable && values.user == User.uid ? '<div class="item-delete-button"></div>' : '';
                }
            }
        );
            
        this.itemSelector = 'div.item';
        this.callParent(arguments);
    },
    verify: function(reports) {
        var i;
        
        for (i=0; i < reports.length; i++) {
            if (i % 2) {
                reports[i]['step'] = 1;
            } else {
                reports[i]['step'] = 0;
            }
        }
        
        return reports;
    },
    show: function(left, top, reports, type) {
        // If reports not found for shared or personal types, we should display a non found notification.
        if (type == 'shared') {
            this.callParent(arguments);
            this.showSharedReports(left, top);
            return;
        } else if (type == 'personal') {
            this.callParent(arguments);
            this.showPersonalReports(left, top);
            return;
        }
        
        // Place a needed reports to store
        this.store.loadData(this.verify(reports));
        
        // Align and show component
        this.align(left, top);
        this.callParent(arguments);
        
        // Invalidate scrolling in component
        this.invalidateScroller();
    },
    hide: function() {
        this.callParent(arguments);
    },
    align: function(left, top) {
        this.setPosition(left, top, false);
    },
    showSharedReports: function(left, top) {
        // Empty previosly setted reports (by another category)
        this.store.loadData(new Array());
        this.setHeight('auto');
        
        // Show & align loader
        this.getEl().update('<div class="w-reports-picker-selection-loader"></div>');
        this.align(left, top);
    },
    onShowSharedReports: function(reports) {
        if (reports.length == 0) {
            this.getEl().update(sprintf('<div class="w-reports-picker-selection-empty">%(noSharedReports)s</div>', { noSharedReports : _t('No result') }));
            return;
        }
        
        // Place fetched reports to store
        this.store.loadData(this.verify(reports));
        
        // Invalidate scrolling in component
        this.invalidateScroller();
    },
    showPersonalReports: function(left, top) {
        // Empty previosly setted reports (by another category)
        this.store.loadData(new Array());
        this.setHeight('auto');
        
        // Show & align loader
        this.getEl().update('<div class="w-reports-picker-selection-loader"></div>');
        this.align(left, top);
    },
    onShowPersonalReports: function(reports) {
        if (reports.length == 0) {
            this.getEl().update(sprintf('<div class="w-reports-picker-selection-empty">%(noSavedReports)s</div>', { noSavedReports : _t('No result') }));
            return;
        }
        
        // Place fetched reports to store
        this.store.loadData(this.verify(reports));
        
        // Invalidate scrolling in component
        this.invalidateScroller();
    },
    
    invalidateScroller: function() {
        var pos = this.getPosition();
        var offset = Ext.getBody().getHeight() - (pos[1] + this.getHeight());
        var height = Ext.getBody().getHeight() - pos[1] - 20;
        var container = this.getEl().first('div.w-reports-picker-selection-container');
        
        if (offset < 20) {
            this.setHeight(height);
            
            container.setHeight(height - 2);
            container.addCls('w-reports-picker-selection-overflow');
        } else {
            this.setHeight('auto');
            
            container.setHeight('auto');
            if (container.hasCls('w-reports-picker-selection-overflow')) {
                container.removeCls('w-reports-picker-selection-overflow');
            }
        }
    },
    onItemClick: function(record, item, index, event) {
        var eventTarget = Ext.get(event.target);

        if (eventTarget.hasCls('item-delete-button')) {
            this.fireEvent('deleteReport', record);
            return;
        }

        this.fireEvent('selectReport', record);
    }
});
