Ext.define('App.view.viewer.Table', {
    extend: 'Ext.grid.Panel',
    alias : 'widget.viewerTable',
    scroll: 'vertical',
    frame: true,
    verticalScrollerType: 'viewerTableScroller',
    invalidateScrollerOnRefresh: false,
    loadMask: true,
    viewType: 'viewerTableView',
    viewConfig: {
        chunker: App.view.viewer.table.Chunker,
        stripeRows: true
    },
    cls: 'w-viewer-table',
    initComponent: function() {
        // Creates a default store
        this.simpleStore = Ext.data.StoreManager.lookup('SimpleEvents');
        this.simpleStore.on('sortchange', this.onStoreSortChange, this);
        this.groupedStore = Ext.data.StoreManager.lookup('GroupedEvents');
        this.groupedStore.on('sortchange', this.onStoreSortChange, this);

        this.store = this.simpleStore;
        this.storeType = 'simple';

        this.horizontalScroller = false;

        // Creates a default columns
        this.columns = new Array();

        // Creates a summary view
        this.summaryCt = Ext.create('App.view.viewer.table.Summary', {
            simpleStore: this.simpleStore,
            groupedStore: this.groupedStore
        });

        this.dockedItems = [
            this.summaryCt
        ]

        this.callParent(arguments);

        this.on('cellclick', function(tthis, td, cellIndex, record, tr, rowIndex, e, eOpts){
            if (e && e.target && e.target.tagName == 'A' && Ext.fly(e.target).hasCls('morerecords')){
                var recordsStr = Ext.fly(e.target).getAttribute('records');
                if(recordsStr && recordsStr != ''){
                    var records = App.utility.File.parseRecordings(recordsStr);

                    var hrmlRecords = '';
                    var pbxid = record.get('PBX');
                    if (pbxid){
                        pbxid = pbxid.replace(/.*\((\w+)\)/,'$1').trim();
                    }
                    for(var i=0; i < records.length; i++){

                        var date = Ext.Date.format(records[i].date, 'd/m/Y H:i:s');
                        for(var j=0; j < records[i].files; j++){
                            var url = null;
                            var userName = '';
                            if (records[i].source == record.get('FROM')){
                                if (record.get('FROM_USER'))
                                    userName = record.get('FROM_USER')+' ('+record.get('FROM')+')' ;
                                else
                                    userName = record.get('FROM_USER');
                            } else if (records[i].source == record.get('TO')) {
                                if (record.get('TO_USER'))
                                    userName = record.get('TO_USER')+' ('+record.get('TO')+')' ;
                                else
                                    userName = record.get('TO');

                            }
                            if(records[i].filename.indexOf('-{n}') > -1){
                                url = '/spoolview/recordings/'+records[i].filename.replace('{n}', j);
                            }else{
                                url = '/spoolview/recordings/'+records[i].filename+'-'+j+'.wav';
                            }
                            if (pbxid && Serial != pbxid){
                                //Download data from another pbx
                                url = '/collaboration/lua/fileproxy.lua?serial='+pbxid+'&path='+url;
                            }
                            hrmlRecords += "<tr>" +
                                                "<td class='w-recording-username'>"+userName+"</td>" +
                                                "<td class='w-recording-filename'>"+date+" "+_t('Record part')+" "+(j+1)+"</td>" +
                                                "<td class='w-recording-link'>" +
                                                    "<a class='w-download-attach w-recording fa fa-file-audio-o' href='../.." + url + "' target='_blank' title='"+_t('Download recording')+"'></a>" +
                                                "</td>" +
                                            "</tr>";
                        }
                    }
                    var html="<table class='w-recording-table'>" +
                            "" + hrmlRecords + "" +
                            "</table>";

                    Ext.create('Ext.window.Window', {
                        title: _t('Records'),
                        maxHeight: 500,
                        width: 400,
                        layout: 'fit',
                        modal: true,
                        items: {
                            xtype: 'panel',
                            padding: 10,
                            style: 'background-color:white',
                            border: false,
                            html: html
                        }
                    }).show();
                }
            }

        });
    },

    /**
     *
     */
    clearColumns: function(identies, type, sortersColumns) {
        var i;
        var typeChange = false;

        // Verify store
        if (this.type != type) {
            typeChange = true;

            if (type == 'simple') {
                this.type = 'simple';

                this.bindStore(this.simpleStore);
                this.verticalScroller.reconfigure(this.simpleStore);
            } else if (type == 'grouped') {
                this.type = 'grouped';

                this.bindStore(this.groupedStore);
                this.verticalScroller.reconfigure(this.groupedStore);
            }
        }

        // Verify columns
        var columns = new Array(), column;
        var groupingsCount = 0;
        this.headerCt.removeAll();
        console.log("identies", identies);

        if (typeChange) {

            for (i=0; i < identies.length; i++) {
                column = this.createColumnByIdenty(identies[i]);
                columns.push(column);
            }

            // define is results is a simple gropings sort
            for (i=0; i < columns.length; i++) {
                if (columns[i]['grouped']) {
                    groupingsCount++;
                }
            }

            if (groupingsCount > 1) {
                this.sortableColumns = false;
            } else {
                this.sortableColumns = true;
            }

            // reset sort state for available columns
            for (i=0; i < columns.length; i++) {
                columns[i]['sortable'] = this.sortableColumns;
            }

            this.headerCt.suspendLayout = true;
            this.headerCt.removeAll();
            this.headerCt.add(columns);
        } else {
            var a, c;
            var len = this.headerCt.getColumnCount();

            // suspend relayouting for each change
            this.headerCt.suspendLayout = true;

            // define is results is a simple gropings sort
            for (i=0; i < identies.length; i++) {
                for (c=0; c < len; c++) {
                    column = this.headerCt.getHeaderAtIndex(c);

                    if (column['identy'] == identies[i] && column['grouped']) {
                        groupingsCount++;
                        break;
                    }
                }
            }

            if (groupingsCount > 1) {
                this.sortableColumns = false;
            } else {
                this.sortableColumns = true;
            }

            // show, hide and reset sort state for available columns
            for (i=0; i < len; i++) {
                a = false;
                column = this.headerCt.getHeaderAtIndex(i);
                if (sortersColumns && sortersColumns.field == column.dataIndex) {
                    column.setSortState(sortersColumns.dest, false, true);
                } else {
                    column.setSortState(null);
                }
                column.sortable = this.sortableColumns;

                for (c=0; c < identies.length; c++) {
                    if (column['identy'] == identies[c]) {
                        a = true;
                    }
                }

                if (!a) {
                    column.hide();
                } else {
                    if (!column.isVisible()) {
                        column.show();
                    }
                }
            }

            // insert new columns
            var index, prev;

            for (i=0; i < identies.length; i++) {
                a = false;
                len = this.headerCt.getColumnCount();

                for (c=0; c < len; c++) {
                    column = this.headerCt.getHeaderAtIndex(c);

                    if (identies[i] == column['identy']) {
                        a = true;
                        break;
                    }
                }

                if (!a) {
                    index = 0;

                    // verify insert index
                    if (i > 0) {
                        prev = identies[i - 1];

                        for (c=0; c < len; c++) {
                            column = this.headerCt.getHeaderAtIndex(c);

                            if (prev == column['identy']) {
                                index = c + 1;
                                break;
                            }
                        }
                    }

                    column = this.createColumnByIdenty(identies[i]);

                    this.headerCt.add(index, column);
                }
            }

            // order grouped columns
            len = this.headerCt.getColumnCount();

            var orders = new Array();

            for (i=0; i < identies.length; i++) {
                for (c=0; c < len; c++) {
                    column = this.headerCt.getHeaderAtIndex(c);

                    if (column['identy'] == identies[i] && column['grouped']) {
                        orders.push(column['identy']);
                        break;
                    }
                }
            }

            for (i=0; i < orders.length; i++) {
                for (c=0; c < len; c++) {
                    column = this.headerCt.items.getAt(c);

                    if (column['identy'] == orders[i]) {
                        if (c != i) {
                            this.headerCt.move(c, i);
                        }

                        break;
                    }
                }
            }

            // Do layout for visible changes
            this.headerCt.suspendLayout = false;
            this.headerCt.doLayout();
        }

        this.summaryCt.reconfigure(this.headerCt.items.getRange());

        this.headerCt.hide();
        this.summaryCt.hide();
        this.headerCt.doLayout();
        this.summaryCt.doLayout();
        // ...
        this.store.on('guaranteedrange', this.onClearedStoreGuaranteedRange, this, {single: true});
    },

    /**
     *
     */
    clearStore: function() {
        console.log('table viewer', 'clearStore');

        this.store.removeAll();
        this.store.prefetchData.clear();
        this.store.guaranteedStart = undefined;
        this.store.guaranteedEnd = undefined;
        this.store.requestEnd = undefined;
        this.store.requestSeed = undefined;
        this.store.requestStart = undefined;
        this.store.totalCount = this.store.pageSize;
    },

    invalidateScrollerProper: function() {
        var scroller = this.verticalScroller;
        var scrollEl = scroller.scrollEl;

        if (scrollEl) {
            scrollEl.removeAllListeners();
            scroller.mon(scrollEl, 'scroll', scroller.onElScroll, scroller);
        }

        this.invalidateScroller();
    },

    /**
     * Fires when store is loaded first time (avoid paging)
     */
    onClearedStoreGuaranteedRange: function() {
        console.log('table viewer', 'onClearedStoreGuaranteedRange');
        this.clearView();

        this.headerCt.show();
        this.summaryCt.show();
        this.headerCt.doLayout();
        this.summaryCt.doLayout();
    },

    onStoreSortChange: function() {
        this.clearView();
    },

    /**
     *
     */
    clearView: function() {
        console.log('table viewer', 'clearView');

        var view = this.getTableView();

        if (view.el) {
            view.el.dom.scrollTop = 0;
        }

        if (!this.verticalScroller.panel) {
            this.verticalScroller.panel = this.verticalScroller.getPanel();
        }

        if (this.verticalScroller.scrollEl) {
            this.verticalScroller.scrollEl.dom.scrollTop = 0;
        }

        this.invalidateScrollerProper();

        return true;
    },

    getSummaryView: function() {
        return this.summaryCt;
    },

    getTableView: function() {
        return this.down('tableview');
    },

    createColumnByIdenty: function(identy) {
        var column;

        switch (identy) {
            // Simple Columns
            case 'NUMBERER':
                column = this.createNumbererColumn();
                break;
            case 'DATE':
                column = this.createDateColumn();
                break;
            case 'FROM':
                column = this.createFromColumn();
                break;
            case 'FROM_GROUP':
                column = this.createFromGroupColumn();
                break;
            case 'TO':
                column = this.createToColumn();
                break;
            case 'TO_GROUP':
                column = this.createToGroupColumn();
                break;
            case 'DURATION':
                column = this.createDurationColumn();
                break;
            case 'BILL':
                column = this.createBillColumn();
                break;
            case 'RING':
                column = this.createRingColumn();
                break;
            case 'COST':
                column = this.createCostColumn();
                break;
            case 'DIRECTION':
                column = this.createDirectionColumn();
                break;
            case 'STATUS':
                column = this.createStatusColumn();
                break;
            case 'AREA':
                column = this.createAreaColumn();
                break;
            case 'CLASS':
                column = this.createClassColumn();
                break;
            case 'TRUNK':
                column = this.createTrunkColumn();
                break;
            case 'PBX':
                column = this.createPbxColumn();
                break;
            case 'TAG':
                column = this.createTagColumn();
                break;
            case 'SERVICE':
                column = this.createServiceColumn();
                break;
            case 'USERFIELD':
                column = this.createUserfieldColumn();
                break;
            case 'FAXDATA':
                column = this.createFaxdataColumn();
                break;

            // Grouped Columns
            case 'TOTAL_EVENTS':
                column = this.createTotalEventsColumn();
                break;
            case 'TOTAL_LOCAL_EVENTS':
                column = this.createTotalLocalEvents();
                break;
            case 'TOTAL_INCOMING_EVENTS':
                column = this.createTotalIncomingEvents();
                break;
            case 'TOTAL_OUTGOING_EVENTS':
                column = this.createTotalOutgoingEvents();
                break;
            case 'TOTAL_ANSWERED_EVENTS':
                column = this.createTotalAnsweredEvents();
                break;
            case 'TOTAL_ANSWERED_WITHIN_XX_EVENTS':
                column = this.createTotalAnsweredWithinXXEvents();
                break;
            case 'TOTAL_ANSWERED_OVER_XX_EVENTS':
                column = this.createTotalAnsweredOverXXEvents();
                break;
            case 'TOTAL_NOT_ANSWERED_EVENTS':
                column = this.createTotalNotAnsweredEvents();
                break;
            case 'TOTAL_CALL_EVENTS':
                column = this.createTotalCallEvents();
                break;
            case 'TOTAL_FAX_EVENTS':
                column = this.createTotalFaxEvents();
                break;
            case 'TOTAL_VOICEMAIL_EVENTS':
                column = this.createTotalVoiceMailEvents();
                break;
            case 'TOTAL_BILL':
                column = this.createTotalBill();
                break;
            case 'AVG_BILL':
                column = this.createAvgBill();
                break;
            case 'TOTAL_LOCAL_BILL':
                column = this.createTotalLocalBill();
                break;
            case 'AVG_LOCAL_BILL':
                column = this.createAvgLocalBill();
                break;
            case 'TOTAL_INCOMING_BILL':
                column = this.createTotalIncomingBill();
                break;
            case 'AVG_INCOMING_BILL':
                column = this.createAvgIncomingBill();
                break;
            case 'TOTAL_OUTGOING_BILL':
                column = this.createTotalOutgoingBill();
                break;
            case 'AVG_OUTGOING_BILL':
                column = this.createAvgOutgoingBill();
                break;

            case 'TOTAL_RING':
                column = this.createTotalRingColumn();
                break;
            case 'AVG_RING':
                column = this.createAvgRingColumn();
                break;

            case 'TOTAL_COST':
                column = this.createTotalCostColumn();
                break;
            case 'TOTAL_CALL_COST':
                column = this.createTotalCallCostColumn();
                break;
            case 'TOTAL_FAX_COST':
                column = this.createTotalFaxCostColumn();
                break;
            case 'AVG_COST':
                column = this.createAvgCostColumn();
                break;
            case 'AVG_CALL_COST':
                column = this.createAvgCallCostColumn();
                break;
            case 'AVG_FAX_COST':
                column = this.createAvgFaxCostColumn();
                break;

            // Grouped Columns
            case 'GROUP_HOUR':
                column = this.createGroupHourColumn();
                break;
            case 'GROUP_HOUR_OF_DAY':
                column = this.createGroupHourOfDayColumn();
                break;
            case 'GROUP_DAY':
                column = this.createGroupDayColumn();
                break;
            case 'GROUP_DAY_OF_WEEK':
                column = this.createGroupDayOfWeekColumn();
                break;
            case 'GROUP_WEEK':
                column = this.createGroupWeekColumn();
                break;
            case 'GROUP_MONTH':
                column = this.createGroupMonthColumn();
                break;
            case 'GROUP_MONTH_OF_YEAR':
                column = this.createGroupMonthOfYearColumn();
                break;
            case 'GROUP_YEAR':
                column = this.createGroupYearColumn();
                break;
            case 'GROUP_USER':
                column = this.createGroupUserColumn();
                break;
            case 'GROUP_CLIENT':
                column = this.createGroupClientColumn();
                break;
            case 'GROUP_COMPANY':
                column = this.createGroupCompanyColumn();
                break;
            case 'GROUP_GROUP':
                column = this.createGroupGroupColumn();
                break;
            case 'GROUP_NUMBER':
                column = this.createGroupNumberColumn();
                break;
            case 'GROUP_SERVICE':
                column = this.createGroupServiceColumn();
                break;
            case 'GROUP_DIRECTION':
                column = this.createGroupDirectionColumn();
                break;
            case 'GROUP_TRUNK':
                column = this.createGroupTrunkColumn();
                break;
            case 'GROUP_TAG':
                column = this.createGroupTagColumn();
                break;
            case 'GROUP_PBX':
                column = this.createGroupPbxColumn();
                break;
            case 'GROUP_COUNTRY':
                column = this.createGroupCountryColumn();
                break;
            case 'GROUP_CLASS':
                column = this.createGroupClassColumn();
                break;

            case 'GROUP_USER_COMM_FROM':
                column = this.createGroupUserCommFromColumn();
                break;
            case 'GROUP_USER_COMM_TO':
                column = this.createGroupUserCommToColumn();
                break;
            case 'GROUP_GROUP_COMM_FROM':
                column = this.createGroupGroupCommFromColumn();
                break;
            case 'GROUP_GROUP_COMM_TO':
                column = this.createGroupGroupCommToColumn();
                break;
        }


        return column;
    },

    createNumbererColumn: function() {
        var self = this;
        var column = Ext.create('Ext.grid.RowNumberer', {
            identy: 'NUMBERER',
            width: 50,
            menuDisabled: true,
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_EVENTS']);
            }
        });

        return column;
    },

    createDateColumn: function() {
        return Ext.create('App.view.viewer.table.Column', {
            identy: 'DATE',
            header: _t('Date'),
            dataIndex: 'DATE',
            width: 125,
            menuDisabled: true,
            renderer: function(value) {
                return Ext.Date.format(Ext.Date.parse(value.substr(0, 19), "Y-m-d H:i:s"), User.getDateTimeFormat());
            }
        });
    },
    createDurationColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'DURATION',
            header: _t('Duration'),
            dataIndex: 'DURATION',
            width: 60,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(parseInt(summaries['TOTAL_DURATION']));
            }
        });

        return column;
    },
    createFromColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'FROM',
            header: _t('From'),
            dataIndex: 'FROM',
            // width: 125,
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                var name = record.get('FROM_USER');
                var fromName = record.get('FROM_NAME');
                if (fromName) {
                    name = fromName;
                }

                name = hide_acl_name(name, value);
                value = hide_acl_digits(value);

                if (name) {
                    if ( !this.skipTags )
                        return name +' (<span style="color:#424242;">'+ Ext.util.Format.htmlEncode(value) +'</span>)';
                    else
                        return name +' ('+ Ext.util.Format.htmlEncode(value) +')';

                }
                return value;
            }
        });

        return column;
    },
    createFromGroupColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'FROM_GROUP',
            header: _t('Group'),
            dataIndex: 'FROM_GROUP',
            width: 75,
            menuDisabled: true,
            renderer: function( value, row, record ) {
                if (value) {
                    return Ext.util.Format.htmlEncode(value);
                }
                return _t('Other');
            }
        });

        return column;
    },
    createToColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TO',
            header: _t('To'),
            dataIndex: 'TO',
            // width: 125,
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                var name = record.get('TO_USER');
                var toName = record.get('TO_NAME');
                if (toName) {
                    name = toName;
                }

                name = hide_acl_name(name, value);
                value = hide_acl_digits(value);

                if (name) {
                    if ( !this.skipTags )
                        return name +' (<span style="color:#424242;">'+ Ext.util.Format.htmlEncode(value) +'</span>)';
                    else
                        return name +' ('+ value +')';
                }
                return value;
            }
        });

        return column;
    },
    createToGroupColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TO_GROUP',
            header: _t('Group'),
            dataIndex: 'TO_GROUP',
            width: 75,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (value) {
                    return Ext.util.Format.htmlEncode(value);
                }
                return _t('Other');
            }
        });

        return column;
    },
    createAreaColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AREA',
            header: _t('Country'),
            dataIndex: 'COUNTRY',
            width: 80,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (value) {
                    var area = record.get('COUNTRY_AREA');
                    return value +", "+ area;
                }
                return _t('Other');
            }
        });

        return column;
    },
    createClassColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'CLASS',
            header: _t('Call Class'),
            dataIndex: 'CLASS',
            width: 80,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatCallClass(value, this.skipTags);
            }
        });

        return column;
    },
    createCostColumn: function() {
        var grade = User.grade_of_accuracy || 2;
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'COST',
            header: _t('Cost'),
            dataIndex: 'COST',
            width: 65,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatWithAccuracyGrade(value, grade) /*+  (this.skipTags?'' : ' &euro;')*/;
            },
            exportRenderer: function(value, row, record) {
                return parseFloat(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatWithAccuracyGrade(summaries['TOTAL_COST'], grade) /*+ (summaries.skipTags?'' : ' &euro;')*/;
            }
        });

        return column;
    },
    createDirectionColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'DIRECTION',
            header: _t('Direction'),
            dataIndex: 'DIRECTION',
            width: 60,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatDirection(value, this.skipTags);
            }
        });

        return column;
    },
    createStatusColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'STATUS',
            header: _t('Status'),
            dataIndex: 'BILL',
            width: 60,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatStatus(value, this.skipTags);
            }
        });

        return column;
    },
    createBillColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'BILL',
            header: _t('Talk time') + ' ('+_t('Seconds short')+')',
            dataIndex: 'BILL',
            width: 90,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatBill(value);
            },
            exportRenderer: function(value, row, record) {
                return parseInt(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(parseInt(summaries['TOTAL_BILL']));
            }
        });

        return column;
    },
    createRingColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'RING',
            header: _t('Ring Time') + ' ('+_t('Seconds short')+')',
            dataIndex: 'RING',
            width: 90,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatBill(value);
            },
            exportRenderer: function(value, row, record) {
                return parseInt(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(parseInt(summaries['TOTAL_RING']));
            }
        });

        return column;
    },
    createTrunkColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TRUNK',
            header: _t('Trunk'),
            dataIndex: 'TRUNK',
            width: 80,
            menuDisabled: true,
            renderer: function( value, row, record ) {
                if (value) {
                    return Ext.util.Format.htmlEncode(value);
                }
                return _t('Other');
            }
        });

        return column;
    },
    createPbxColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'PBX',
            header: _t('Pbx'),
            dataIndex: 'PBX',
            width: 125,
            menuDisabled: true,
            renderer: function( value, row, record ) {
                if (value) {
                    return value;
                }
                return _t('Other');
            }
        });

        return column;
    },
    createTagColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TAG',
            header: _t('Tag'),
            dataIndex: 'TAG',
            width: 80,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (value) {
                    return Ext.util.Format.htmlEncode(value);
                }
                return '-';
            }
        });

        return column;
    },
    createServiceColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'SERVICE',
            header: _t('Service'),
            dataIndex: 'SERVICE',
            width: 80,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return (this.skipTags ? '' :
                         App.utility.File.userDataLink(record) + App.utility.File.faxDataLink(record)
                       ) + App.utility.NamesFormatter.formatService(value, this.skipTags);
            }
        });

        return column;
    },
    createUserfieldColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'USERFIELD',
            header: '',
            hidden: true,
            dataIndex: 'USERFIELD',
            width: 80,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return Ext.util.Format.htmlEncode(value);
            }
        });

        return column;
    },
    createFaxdataColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'FAXDATA',
            header: '',
            hidden: true,
            dataIndex: 'FAXDATA',
            width: 80,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return Ext.util.Format.htmlEncode(value);
            }
        });

        return column;
    },
    createTotalEventsColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_EVENTS',
            header: _t('Events'),
            dataIndex: 'TOTAL_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_EVENTS']);
            }
        });

        return column;
    },
    createTotalLocalEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_LOCAL_EVENTS',
            header: _t('Local events'),
            dataIndex: 'TOTAL_LOCAL_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_LOCAL_EVENTS']);
            }
        });

        return column;
    },
    createTotalIncomingEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_INCOMING_EVENTS',
            header: _t('Inc. events'),
            dataIndex: 'TOTAL_INCOMING_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_INCOMING_EVENTS']);
            }
        });

        return column;
    },
    createTotalOutgoingEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_OUTGOING_EVENTS',
            header: _t('Out. events'),
            dataIndex: 'TOTAL_OUTGOING_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_OUTGOING_EVENTS']);
            }
        });

        return column;
    },
    createTotalAnsweredEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_ANSWERED_EVENTS',
            header: _t('Answ. events'),
            dataIndex: 'TOTAL_ANSWERED_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_ANSWERED_EVENTS']);
            }
        });

        return column;
    },
    createTotalAnsweredWithinXXEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_ANSWERED_WITHIN_XX_EVENTS',
            header: sprintf(_t('Answ. events within %(seconds)s'),{seconds:User.sla_period}),
            dataIndex: 'TOTAL_ANSWERED_WITHIN_XX_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_ANSWERED_WITHIN_XX_EVENTS']);
            }
        });

        return column;
    },
    createTotalAnsweredOverXXEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_ANSWERED_OVER_XX_EVENTS',
            header: sprintf(_t('Answ. events over %(seconds)s'),{seconds:User.sla_period}),
            dataIndex: 'TOTAL_ANSWERED_OVER_XX_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_ANSWERED_OVER_XX_EVENTS']);
            }
        });

        return column;
    },
    createTotalNotAnsweredEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_NOT_ANSWERED_EVENTS',
            header: _t('Unansw. events'),
            dataIndex: 'TOTAL_NOT_ANSWERED_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_NOT_ANSWERED_EVENTS']);
            }
        });

        return column;
    },

    createTotalCallEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_CALL_EVENTS',
            header: _t('Call events'),
            dataIndex: 'TOTAL_CALL_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_CALL_EVENTS']);
            }
        });

        return column;
    },
    createTotalFaxEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_FAX_EVENTS',
            header: _t('Fax events'),
            dataIndex: 'TOTAL_FAX_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_FAX_EVENTS']);
            }
        });

        return column;
    },
    createTotalVoiceMailEvents: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_VOICEMAIL_EVENTS',
            header: _t('Voicemail events'),
            dataIndex: 'TOTAL_VOICEMAIL_EVENTS',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatNumber(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatNumber(summaries['TOTAL_VOICEMAIL_EVENTS']);
            }
        });

        return column;
    },
    createTotalBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_BILL',
            header: _t('Talk time'),
            dataIndex: 'TOTAL_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['TOTAL_BILL']);
            }
        });

        return column;
    },
    createAvgBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_BILL',
            header: _t('Avg. Talk time'),
            dataIndex: 'AVG_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['AVG_BILL']);
            }
        });

        return column;
    },
    createTotalLocalBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_LOCAL_BILL',
            header: _t('Local Talk time'),
            dataIndex: 'TOTAL_LOCAL_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['TOTAL_LOCAL_BILL']);
            }
        });

        return column;
    },
    createAvgLocalBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_LOCAL_BILL',
            header: _t('Avg. Local Talk time'),
            dataIndex: 'AVG_LOCAL_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['AVG_LOCAL_BILL']);
            }
        });

        return column;
    },
    createTotalIncomingBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_INCOMING_BILL',
            header: _t('Inc. Talk time'),
            dataIndex: 'TOTAL_INCOMING_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['TOTAL_INCOMING_BILL']);
            }
        });

        return column;
    },
    createAvgIncomingBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_INCOMING_BILL',
            header: _t('Avg. Inc. Talk time'),
            dataIndex: 'AVG_INCOMING_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['AVG_INCOMING_BILL']);
            }
        });

        return column;
    },
    createTotalOutgoingBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_OUTGOING_BILL',
            header: _t('Out. Talk time'),
            dataIndex: 'TOTAL_OUTGOING_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['TOTAL_OUTGOING_BILL']);
            }
        });

        return column;
    },
    createAvgOutgoingBill: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_OUTGOING_BILL',
            header: _t('Avg. Out. Talk time'),
            dataIndex: 'AVG_OUTGOING_BILL',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['AVG_OUTGOING_BILL']);
            }
        });

        return column;
    },

    createTotalRingColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_RING',
            header: _t('Ring Time'),
            dataIndex: 'TOTAL_RING',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['TOTAL_RING']);
            }
        });

        return column;
    },
    createAvgRingColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_RING',
            header: _t('Avg. Ring Time'),
            dataIndex: 'AVG_RING',
            flex: 1,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatHoursTime(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatHoursTime(summaries['AVG_RING']);
            }
        });

        return column;
    },
    createTotalCostColumn: function() {
        var grade = User.grade_of_accuracy || 2;
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_COST',
            header: _t('Cost'),
            dataIndex: 'TOTAL_COST',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatWithAccuracyGrade(value, grade);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatWithAccuracyGrade(summaries['TOTAL_COST'], grade);
            }
        });

        return column;
    },
    createTotalCallCostColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_CALL_COST',
            header: _t('Call cost'),
            dataIndex: 'TOTAL_CALL_COST',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatFloat(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatFloat(summaries['TOTAL_CALL_COST']);
            }
        });

        return column;
    },
    createTotalFaxCostColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'TOTAL_FAX_COST',
            header: _t('Fax cost'),
            dataIndex: 'TOTAL_FAX_COST',
            flex: 1,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatFloat(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatFloat(summaries['TOTAL_FAX_COST']);
            }
        });

        return column;
    },
    createAvgCostColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_COST',
            header: _t('Avg. Cost'),
            dataIndex: 'AVG_COST',
            flex: 1,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatFloat(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatFloat(summaries['AVG_COST']);
            }
        });

        return column;
    },
    createAvgCallCostColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_CALL_COST',
            header: _t('Avg. Call Cost'),
            dataIndex: 'AVG_CALL_COST',
            flex: 1,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatFloat(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatFloat(summaries['AVG_CALL_COST']);
            }
        });

        return column;
    },
    createAvgFaxCostColumn: function() {
        var self = this;
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'AVG_FAX_COST',
            header: _t('Avg. Fax Cost'),
            dataIndex: 'AVG_FAX_COST',
            flex: 1,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatFloat(value);
            },
            summaryRenderer: function(summaries) {
                return App.utility.NamesFormatter.formatFloat(summaries['AVG_FAX_COST']);
            }
        });

        return column;
    },
    createGroupHourColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_HOUR',
            header: _t('Hour'),
            dataIndex: 'HOUR',
            width: 115,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatGroupingByHour(value);
            }
        });

        return column;
    },
    createGroupHourOfDayColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_HOUR_OF_DAY',
            header: _t('Hour of day'),
            dataIndex: 'HOUR_OF_DAY',
            width: 115,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatHourOfDay(value);
            }
        });

        return column;
    },
    createGroupDayColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_DAY',
            header: _t('Day'),
            dataIndex: 'DAY',
            width: 80,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatGroupingByDay(value);
            }
        });

        return column;
    },
    createGroupDayOfWeekColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_DAY_OF_WEEK',
            header: _t('Day of week'),
            dataIndex: 'DAY_OF_WEEK',
            width: 130,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatDayOfWeek(value);
            }
        });

        return column;
    },
    createGroupWeekColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_WEEK',
            header: _t('Week'),
            dataIndex: 'WEEK',
            width: 150,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatGroupingByWeek(value);
            }
        });

        return column;
    },
    createGroupMonthColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_MONTH',
            header: _t('Month'),
            dataIndex: 'MONTH',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatByMonth(value);
            }
        });

        return column;
    },
    createGroupMonthOfYearColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_MONTH_OF_YEAR',
            header: _t('Month of year'),
            dataIndex: 'MONTH_OF_YEAR',
            width: 130,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                return App.utility.NamesFormatter.formatMonthOfYear(value);
            }
        });

        return column;
    },
    createGroupYearColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_YEAR',
            header: _t('Year'),
            dataIndex: 'YEAR',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return value;
            }
        });

        return column;
    },
    createGroupUserColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_USER',
            header: _t('User'),
            dataIndex: 'USER',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (!value) {
                    return _t('Other');
                }
                var name = record.get('USER_NAME');
                return Ext.util.Format.htmlEncode(name);
            }
        });

        return column;
    },
    createGroupClientColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_CLIENT',
            header: _t('Business partner'),
            dataIndex: 'CLIENT',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (!value) {
                    return _t('Other');
                }
                var name = record.get('CLIENT_NAME');
                return Ext.util.Format.htmlEncode(name);
            }
        });

        return column;
    },
    createGroupCompanyColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_COMPANY',
            header: _t('Company'),
            dataIndex: 'COMPANY',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (!value) {
                    return _t('Other');
                }
                var name = record.get('COMPANY_NAME');
                return Ext.util.Format.htmlEncode(name);
            }
        });

        return column;
    },
    createGroupGroupColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_GROUP',
            header: _t('Group'),
            dataIndex: 'USER_GROUP',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (!value) {
                    return _t('Other');
                }
                var name = record.get('USER_GROUP_NAME');
                return Ext.util.Format.htmlEncode(name);
            }
        });

        return column;
    },
    createGroupNumberColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_NUMBER',
            header: _t('Number'),
            dataIndex: 'NUMBER',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                value = hide_acl_digits(value);

                if (!value) {
                    return _t('Other');
                }
                return value;
            }
        });

        return column;
    },
    createGroupServiceColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_SERVICE',
            header: _t('Service'),
            dataIndex: 'SERVICE',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatService(value, this.skipTags);
            }
        });

        return column;
    },
    createGroupDirectionColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_DIRECTION',
            header: _t('Direction'),
            dataIndex: 'DIRECTION',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatGroupDirection(value);
            }
        });

        return column;
    },
    createGroupTrunkColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_TRUNK',
            header: _t('Trunk'),
            dataIndex: 'TRUNK',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (!value) {
                    return _t('Other');
                }
                var name = record.get('TRUNK_NAME');
                return Ext.util.Format.htmlEncode(name);
            }
        });

        return column;
    },
    createGroupTagColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_TAG',
            header: _t('Tag'),
            dataIndex: 'TAG',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (!value) {
                    return '-';
                }
                var name = record.get('TAG_NAME');
                return Ext.util.Format.htmlEncode(name);
            }
        });

        return column;
    },
    createGroupPbxColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_PBX',
            header: _t('Pbx'),
            dataIndex: 'PBX',
            width: 120,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return value;
            }
        });

        return column;
    },
    createGroupCountryColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_COUNTRY',
            header: _t('Country'),
            dataIndex: 'COUNTRY_NAME',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                if (!value) {
                    return _t('Other');
                }
                return value;
            }
        });

        return column;
    },
    createGroupClassColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_CLASS',
            header: _t('Class'),
            dataIndex: 'CLASS',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value, row, record) {
                return App.utility.NamesFormatter.formatCallClass(value);
            }
        });

        return column;
    },
    createGroupUserCommFromColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_USER_COMM_FROM',
            header: _t('From'),
            dataIndex: 'USER_FROM_NAME',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                if (!value) {
                    return _t('Other');
                }
                return Ext.util.Format.htmlEncode(value);
            }
        });

        return column;
    },
    createGroupUserCommToColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_USER_COMM_TO',
            header: _t('To'),
            dataIndex: 'USER_TO_NAME',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                if (!value) {
                    return _t('Other');
                }
                return Ext.util.Format.htmlEncode(value);
            }
        });

        return column;
    },
    createGroupGroupCommFromColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_GROUP_COMM_FROM',
            header: _t('From'),
            dataIndex: 'GROUP_FROM_NAME',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                if (!value) {
                    return _t('Other');
                }
                return Ext.util.Format.htmlEncode(value);
            }
        });

        return column;
    },
    createGroupGroupCommToColumn: function() {
        var column = Ext.create('App.view.viewer.table.Column', {
            identy: 'GROUP_GROUP_COMM_TO',
            header: _t('To'),
            dataIndex: 'GROUP_TO_NAME',
            width: 100,
            grouped: true,
            menuDisabled: true,
            renderer: function(value) {
                if (!value) {
                    return _t('Other');
                }
                return Ext.util.Format.htmlEncode(value);
            }
        });

        return column;
    },
    formatMinutesTime: function(secs) {
        var divisor_for_minutes = secs % (60 * 60);
        var minutes = Math.floor(divisor_for_minutes / 60);
        var divisor_for_seconds = divisor_for_minutes % 60;
        var seconds = Math.ceil(divisor_for_seconds);

        if (minutes < 10) {
            minutes = "0" + minutes;
        }
        if (seconds < 10) {
            seconds = "0" + seconds;
        }

        return minutes +':'+ seconds;
    }
});
