/**
 * Created by PyCharm.
 * User: Artyom Trityak
 * Date: 1/5/12
 * Time: 5:21 PM
 * To change this template use File | Settings | File Templates.
 */
Ext.define('App.view.viewer.chart.type.DefaultChart', {
    extend: 'Ext.chart.Chart',

    legend: {
        position: 'right'
    },

    options: false,

    constructor: function(options) {
        console.log('construct');
        console.log(options);
        this.options = options || {};
        this.callParent([{theme: 'Category1'}]);
    },

    initComponent: function() {
        var fields = this.createFieldsToStore();
        this.options.fields = fields;
        console.log(fields);

        this.series = this.createSeries(fields, this.options.records.length);
        var axes = this.createAxes(fields);
        if (axes !== false) {
            this.axes = axes;
        }
        this.checkPossibilityToDraw();

        this.store = Ext.create('Ext.data.JsonStore', {
            fields: fields.totalFields,
            data: this.options.records
        });
        this.callParent(arguments);
    },

    createFieldsToStore: function() {
        var xGroupField = false;
        var yFields = [];
        var totalFields = [];
        for (var key in this.options.records[0]) {
            if (key.match(/_CHART_GROUPING$/)) {
                if (this.options.records[0].hasOwnProperty(key + "_NAME")) {
                    xGroupField = key + "_NAME";
                    totalFields.push(key + "_NAME");
                } else {
                    xGroupField = key;
                    totalFields.push(key);
                }
            } else if (key.match(/_CHART_VAL$/)) {
                yFields.push(key);
                totalFields.push({
                    name: key,
                    type: 'float'
                });
            }
        }
        return {
            xGroupField: xGroupField,
            yFields: yFields,
            totalFields: totalFields
        };
    },

    createTip: function() {
        var self = this;
        var tip = {
            trackMouse: true,
            minWidth: 250,
            renderer: function(storeItem, item) {
                console.log(arguments);
                var label = item._yField || '';
                var countSame = 0;
                if (!label){
                    for (var key in storeItem.data) {
                        if (storeItem.data[key] === item.value[1]) {
                            label = key;
                            countSame++;
                        }
                    }
                } else {
                    countSame = 1;
                }
                var fieldHeader = self.getFormattedField(storeItem.get(self.options.fields.xGroupField));
                var itemVal = self._formatTipData(item.value[1]);
                if (countSame === 1) {
                    var tooltip = fieldHeader + " <br/> " + itemVal;
                    var title = App.utility.NamesFormatter.formatText(label);
                } else {
                    tooltip = fieldHeader + " <br/> " + itemVal;
                    title = _t('Chart');
                }
                this.setTitle(title);
                this.update(tooltip);
            }
        };
        return tip;
    },

    _formatTipData: function(value) {
        var type = 'NONE';
        for (var key in this.options.records[0]) {
            if (key.match(/_BILL_/) || key.match(/_RING_/)) {
                type = 'TIME';
                break;
            }
            if (key.match(/_COST_/)) {
                type = 'COST';
                break;
            }
        }
        switch (type) {
            case 'COST':
                var result = App.utility.NamesFormatter.formatFloat(value) + ' \u20ac';
                break;
            case 'TIME':
                result = App.utility.NamesFormatter.formatSecondsShort(value);
                break;
            default:
                result = value;
        }
        return result;
    },

    canDraw: function() {
        return this.options.draw;
    },

    getReason: function() {
        return this.options.drawReason;
    },

    checkPossibilityToDraw: function() {
        this.options.draw = true;
        this.options.drawReason = "";
    },

    getFormattedField: function(value) {
        if (value == '' || value == null || value == undefined) {
            return _t('Other');
        }
        var formattedName = value;
        switch (this.options.fields.xGroupField) {
            case 'GROUP_HOUR_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatGroupingByHour(value);
                break;
            case 'GROUP_DAY_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatGroupingByDay(value);
                break;
            case 'GROUP_HOUR_OF_DAY_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatHourOfDay(value);
                break;
            case 'GROUP_DAY_OF_WEEK_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatDayOfWeek(value);
                break;
            case 'GROUP_WEEK_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatGroupingByWeek(value);
                break;
            case 'GROUP_MONTH_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatByMonth(value);
                break;
            case 'GROUP_MONTH_OF_YEAR_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatMonthOfYear(value);
                break;
            case 'GROUP_CLASS_CHART_GROUPING':
                formattedName = App.utility.NamesFormatter.formatCallClass(value);
                break;
            case 'GROUP_NUMBER_CHART_GROUPING':
                formattedName = hide_acl_digits(value);
                break;

        }
        return formattedName;
    },

    getFormattedLegend: function() {
        var formattedLegend = [];
        for (var i in this.options.fields.yFields) {
            formattedLegend.push(App.utility.NamesFormatter.formatText(this.options.fields.yFields[i]));
        }
        return formattedLegend;
    },
    afterComponentLayout: function(width, height) {
        var me = this;
        if (Ext.isNumber(width) && Ext.isNumber(height)) {
            me.curWidth = width;
            me.curHeight = height;
            me.redraw(true);
        }
        this.callParent(arguments);

        if (!me.layoutDone){
            //disable by default Total from graphs
            //the category "Total events" must not be shown by default
            //Total events is not needed usually and confuses customers because it double events in any graph
            var yField = [];
            me.series.each(function(series, index){
                if (Ext.isArray(series.yField))
                    yField = series.yField;
                else
                    yField.push(series.yField);
            });
            var disableFieldsByDefault = [
                "TOTAL_EVENTS_CHART_VAL",
                "TOTAL_COST_CHART_VAL",
                "TOTAL_BILL_CHART_VAL",
                "TOTAL_LOCAL_BILL_CHART_VAL"
            ];

            for (var i=0; i<disableFieldsByDefault.length; i++){
                var indexOfTotals = yField.indexOf(disableFieldsByDefault[i]);
                if (yField.length > 2 && indexOfTotals>=0) {
                    me.legend.items[indexOfTotals].fireEvent('mouseover',me.legend.items[indexOfTotals]);
                    me.legend.items[indexOfTotals].fireEvent('mousedown',me.legend.items[indexOfTotals]);
                    me.legend.items[indexOfTotals].fireEvent('mouseout',me.legend.items[indexOfTotals]);
                }
            }
            me.layoutDone = true;
        }
    }

});