Ext.define('App.view.viewer.table.Summary', {
    extend: 'Ext.Container',
    dock: 'bottom',
    cls: 'w-viewer-table-summary',
    layout: 'hbox',
    initComponent: function() {
        var self = this;
        
        this.simpleStore.on('guaranteedrange', function() {
            self.onGuaranteedRange.call(self, self.simpleStore);
        }, this);
        this.groupedStore.on('guaranteedrange', function() {
            self.onGuaranteedRange.call(self, self.groupedStore);
        }, this);
        this.callParent(arguments);
    },
    reconfigure: function(columns) {
        var i, item, column;
        
        // suspend relayouting for each change
        this.suspendLayout = true;
        
        // Verify items
        var len = this.items.getCount();
        var offset = columns.length - this.items.getCount();
        
        if (offset != 0) {
            var type = offset > 0 ? 1 : 0;
            var abs = Math.abs(offset);
            
            if (offset > 0) {
                // Create new items
                for (i=0; i < abs; i++) {
                    this.add(this.createColumn());
                }
            } else {
                // Remove items
                for (i=0; i < abs; i++) {
                    this.remove(this.items.getAt(len - i - 1), true);
                }
            }
        }
        
        for (i=0; i < columns.length; i++) {
            column = columns[i];
            item = this.items.getAt(i);
            
            // Verify bindings
            if (!column['summaryBinded']) {
                column.on('resize', this.onHeaderColumnResize, this);
            }
            
            // Clear previosly setted value
            if (item.rendered) {
                item.valueEl.update('');
            }
            
            // Verify visibility
            if (column.isVisible()) {
                item.show();
            } else {
                item.hide();
            }
            
            // Set First & Last cls
            if (i == 0) {
                item.addCls('w-viewer-table-summary-column-first');
            } else if (i == columns.length - 1) {
                item.addCls('w-viewer-table-summary-column-last');
            } else if (item.rendered && (item.getEl().hasCls('w-viewer-table-summary-column-first') || item.getEl().hasCls('w-viewer-table-summary-column-last'))) {
                item.getEl().removeCls('w-viewer-table-summary-column-first');
                item.getEl().removeCls('w-viewer-table-summary-column-last');
            }
            
            // Set Width & Id
            item.columnId = column['id'];
            item.columnRenderer = column['summaryRenderer'];
            item.setWidth(column.getWidth());
        }
        
        // Do layout for visible changes
        this.suspendLayout = false;
        this.doLayout();
    },
    
    createColumn: function() {
        var column = Ext.create('Ext.container.Container', {
            cls: 'w-viewer-table-summary-column',
            renderTpl: [
                '<div class="w-viewer-table-summary-column-inner"></div>'
            ],
            renderSelectors: {
                valueEl: 'div.w-viewer-table-summary-column-inner'
            }
        });
        
        return column;
    },
    
    onGuaranteedRange: function(store) {
        var i, item;
        var len = this.items.getCount();
        var summaries = store.getSummaries();
        
        for (i=0; i < len; i++) {
            item = this.items.getAt(i);
            
            if (Ext.isFunction(item.columnRenderer)) {
                item.valueEl.update(item.columnRenderer(summaries));
            }
        }
    },
    setColumnLoading: function() {
        
    },
    setColumnValue: function() {
        
    },
    onHeaderColumnTransfer: function() {
        console.log('onHeaderColumnTransfer');
    },
    onHeaderColumnRemove: function() {
        console.log('onHeaderColumnRemove');
    },
    onHeaderColumnResize: function(column, width) {
        var i = 0, item; 
        
        for( i=0; i < this.items.length; i++ ) {
            item = this.items.get(i);
            
            if( item['columnId'] == column['id'] ) {
                item.setWidth(width);
            }
        }
    }
});