Ext.define('App.view.viewport.Switcher', {
    extend: 'Ext.view.View',
    alias : 'widget.viewportSwitcher',
    cls: 'w-switcher',
    switchable: false,
    initComponent: function() {
        this.addEvents('change');
        this.store = Ext.create('Ext.data.JsonStore', {
            fields: [
                'id',
                'text',
                'active',
                'side'
            ],
            data: this.verifyMenus(this.menu)
        });
        this.tpl = this.getTplInstance();
        this.itemSelector = 'div.item';
        this.callParent(arguments);
    },
    getTplInstance: function() {
         return new Ext.XTemplate(
            '<tpl for=".">',
                '<div class="item <tpl if="side == 1">item-first</tpl> <tpl if="side == 2">item-last</tpl> <tpl if="side == 3">item-first item-last</tpl>  <tpl if="active == 1">item-active</tpl>"><div class="item-left"><div class="item-right">',
                    '{text}',
                '</div></div></div>',
            '</tpl>'
        );
    },
    verifyMenus: function(menus) {
        var i;
        
        if(menus == undefined)
            return [];
        for (i=0; i < menus.length; i++) {
            if (i == 0) {
                menus[i]['side'] = 1;
            } else if(i == menus.length - 1) {
                menus[i]['side'] = 2;
            } else {
                menus[i]['side'] = 0;
            }
            if (menus.length == 1){
                menus[i]['side'] = 3;
            }
        }
        
        return menus;
    },
    onItemClick: function(record, item, index) {

        if (this.switchable){
            if (record.get('active') == 1)
                record.set('active', 0);
            else
                record.set('active', 1);

            this.fireEvent('change', record.get('id'), record.get('active') == 1);
            return true;
        }

        var i, r;
        
        if (record.get('active') == 1){
            return;
        }
        
        for (i=0; i < this.store.getCount(); i++) {
            r = this.store.getAt(i);
            
            if (r.get('active') == 1) {
                r.set('active', 0);
            }
        }
        
        record.set('active', 1);
        this.fireEvent('change', record.get('id'));
    },
    
    setMenu: function(items) {
        this.store.loadData(this.verifyMenus(items));
    },
    
    /**
     * Sets active item for switcher, if id is false any item will be unactive.
     */
    setActiveItem: function(id) {
        this.clearSelection();
        
        if (id) {
            var item = this.store.getById(id);

            if (item) {
                item.set('active', true);
            }
        }
    },
    
    getActiveItem: function() {
        var active = this.store.queryBy(function(rec){
            return rec.get('active');
        }).first();

        if (active) {
            return active.get('id');
        }

        return false;
    },
    
    /**
     * @private
     */
    clearSelection: function(){
        this.store.each(function(itm){
            if(itm.get('active'))
                itm.set('active', false);
        });
    }
});